@@ -26,6 +26,7 @@ import java.io.StringWriter;
 import java.io.Writer;
 import java.net.URL;
 import java.util.Iterator;
+import java.util.Stack;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.logging.Logger;
@@ -38,6 +39,7 @@ import javax.xml.stream.Location;
 import javax.xml.stream.StreamFilter;
 import javax.xml.stream.XMLInputFactory;
 import javax.xml.stream.XMLOutputFactory;
+import javax.xml.stream.XMLResolver;
 import javax.xml.stream.XMLStreamConstants;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
@@ -59,6 +61,7 @@ import org.w3c.dom.CDATASection;
 import org.w3c.dom.Comment;
 import org.w3c.dom.Document;
 import org.w3c.dom.DocumentFragment;
+import org.w3c.dom.DocumentType;
 import org.w3c.dom.Element;
 import org.w3c.dom.EntityReference;
 import org.w3c.dom.NamedNodeMap;
@@ -91,6 +94,9 @@ public final class StaxUtils {
         "ns7".intern(), "ns8".intern(), "ns9".intern()
     };
     
+    private static int innerElementLevelThreshold = -1;
+    private static int innerElementCountThreshold = -1;
+    
     static {
         int i = 20;
     
@@ -107,6 +113,28 @@ public final class StaxUtils {
         }
         NS_AWARE_INPUT_FACTORY_POOL = new LinkedBlockingQueue<XMLInputFactory>(i);
         OUTPUT_FACTORY_POOL = new LinkedBlockingQueue<XMLOutputFactory>(i);
+        try {
+            String s =  System.getProperty("org.apache.cxf.staxutils.innerElementLevelThreshold",
+                                    "-1");
+            innerElementLevelThreshold = Integer.parseInt(s);
+        } catch (Throwable t) {
+            innerElementLevelThreshold = -1;
+        }
+        if (innerElementLevelThreshold <= 0) {
+            innerElementLevelThreshold = -1;
+        }
+        try {
+            String s =  System.getProperty("org.apache.cxf.staxutils.innerElementCountThreshold",
+                                    "-1");
+            innerElementCountThreshold = Integer.parseInt(s);
+        } catch (Throwable t) {
+            innerElementCountThreshold = -1;
+        }
+        if (innerElementCountThreshold <= 0) {
+            innerElementCountThreshold = -1;
+        }
+        
+        
     }
     
     private StaxUtils() {
@@ -135,8 +163,7 @@ public final class StaxUtils {
     private static XMLInputFactory getXMLInputFactory() {
         XMLInputFactory f = NS_AWARE_INPUT_FACTORY_POOL.poll();
         if (f == null) {
-            f = XMLInputFactory.newInstance();
-            f.setProperty(XMLInputFactory.IS_NAMESPACE_AWARE, true);
+            f = createXMLInputFactory(true);
         }
         return f;
     }
@@ -165,6 +192,16 @@ public final class StaxUtils {
     public static XMLInputFactory createXMLInputFactory(boolean nsAware) {
         XMLInputFactory factory = XMLInputFactory.newInstance();
         factory.setProperty(XMLInputFactory.IS_NAMESPACE_AWARE, nsAware);
+        factory.setProperty(XMLInputFactory.SUPPORT_DTD, Boolean.FALSE);
+        factory.setProperty(XMLInputFactory.IS_REPLACING_ENTITY_REFERENCES, Boolean.FALSE);
+        factory.setProperty(XMLInputFactory.IS_SUPPORTING_EXTERNAL_ENTITIES, Boolean.FALSE);
+        factory.setXMLResolver(new XMLResolver() {
+            public Object resolveEntity(String publicID, String systemID,
+                                        String baseURI, String namespace)
+                throws XMLStreamException {
+                throw new XMLStreamException("Reading external entities is disabled");
+            }
+        });
         return factory;
     }
 
@@ -349,31 +386,56 @@ public final class StaxUtils {
         }
         return false;
     }
+    public static void copy(Source source, OutputStream os) throws XMLStreamException {
+        XMLStreamWriter writer = createXMLStreamWriter(os);
+        try {
+            copy(source, writer);
+        } finally {
+            try {
+                writer.flush();
+            } catch (XMLStreamException ex) {
+                //ignore
+            }
+        }
+    }
     public static void copy(Source source, XMLStreamWriter writer) throws XMLStreamException {
         if (source instanceof SAXSource) {
-            InputSource src = ((SAXSource)source).getInputSource();
-            if (src.getSystemId() == null && src.getPublicId() == null
-                && ((SAXSource)source).getXMLReader() != null) {
-                
-                //OK - reader is OK.  We'll use that out
-                StreamWriterContentHandler ch = new StreamWriterContentHandler(writer);
-                XMLReader reader = ((SAXSource)source).getXMLReader();
-                reader.setContentHandler(ch);
-                try {
+            SAXSource ss = (SAXSource)source;
+            InputSource src = ss.getInputSource();
+            if (src == null || (src.getSystemId() == null && src.getPublicId() == null)) {
+                if (ss.getXMLReader() != null) {
+                    //OK - reader is OK.  We'll use that out
+                    StreamWriterContentHandler ch = new StreamWriterContentHandler(writer);
+                    XMLReader reader = ((SAXSource)source).getXMLReader();
+                    reader.setContentHandler(ch);
                     try {
-                        reader.setFeature("http://xml.org/sax/features/namespaces", true);
-                    } catch (Throwable t) {
-                        //ignore
+                        try {
+                            reader.setFeature("http://xml.org/sax/features/namespaces", true);
+                        } catch (Throwable t) {
+                            //ignore
+                        }
+                        reader.parse(((SAXSource)source).getInputSource());
+                        return;
+                    } catch (Exception e) {
+                        throw new XMLStreamException(e);
                     }
-                    reader.parse(((SAXSource)source).getInputSource());
+                } else if (ss.getInputSource() == null) {
+                    //nothing to copy, just return
                     return;
-                } catch (Exception e) {
-                    throw new XMLStreamException(e);
                 }
             }
        
         }
-        
+
+        if (source instanceof StreamSource) {
+            StreamSource ss = (StreamSource)source;
+            if (ss.getInputStream() == null
+                && ss.getReader() == null
+                && ss.getSystemId() == null) {
+                //nothing to copy, just return
+                return;
+            }
+        }
         XMLStreamReader reader = createXMLStreamReader(source);
         copy(reader, writer);
         reader.close();
@@ -464,8 +526,6 @@ public final class StaxUtils {
             prefix = "";
         }
 
-        
-//        System.out.println("STAXUTILS:writeStartElement : node name : " + local +  " namespace URI" + uri);
         boolean writeElementNS = false;
         
         if (uri != null) {
@@ -618,8 +678,6 @@ public final class StaxUtils {
         String ns = e.getNamespaceURI();
         String localName = e.getLocalName();
 
-       
-//        System.out.println("local name : " + localName + " URI: " + ns + " Prefix :" + prefix);
         if (prefix == null) {
             prefix = "";
         }
@@ -640,8 +698,6 @@ public final class StaxUtils {
                 declareNamespace = false;
             }
         } else {
-//            System.out.println("Calling writeStartElement for local name : " 
-//            + localName + " URI: " + ns + " Prefix :" + prefix);
             writer.writeStartElement(prefix, localName, ns);
         }
 
@@ -659,8 +715,6 @@ public final class StaxUtils {
             }
      
             if ("xmlns".equals(attrPrefix)) {
-//                System.out.println("WriteNamespace is called for prefix : " 
-//                + name + " namespace :" + attr.getNodeValue());
                 writer.writeNamespace(name, attr.getNodeValue());
                 writer.setPrefix(name, attr.getNodeValue());
                 if (name.equals(prefix) && attr.getNodeValue().equals(ns)) {
@@ -746,11 +800,57 @@ public final class StaxUtils {
             }
             break;
         }
+        case Node.DOCUMENT_TYPE_NODE:
+            try {
+                if (((DocumentType)n).getTextContent() != null) {
+                    writer.writeDTD(((DocumentType)n).getTextContent());
+                }
+            } catch (UnsupportedOperationException ex) {
+                //can we ignore?  DOM writers really don't allow this
+                //as there isn't a way to write a DTD in dom
+            }
+            break;
         default:
             throw new IllegalStateException("Found type: " + n.getClass().getName());
         }        
     }
 
+    public static Document read(Source s) throws XMLStreamException {
+        XMLStreamReader reader = createXMLStreamReader(s);
+        try {
+            return read(reader);
+        } finally {
+            try {
+                reader.close();
+            } catch (Exception ex) {
+                //ignore
+            }
+        }
+    }
+    public static Document read(InputStream s) throws XMLStreamException {
+        XMLStreamReader reader = createXMLStreamReader(s);
+        try {
+            return read(reader);
+        } finally {
+            try {
+                reader.close();
+            } catch (Exception ex) {
+                //ignore
+            }
+        }
+    }
+    public static Document read(InputSource s) throws XMLStreamException {
+        XMLStreamReader reader = createXMLStreamReader(s);
+        try {
+            return read(reader);
+        } finally {
+            try {
+                reader.close();
+            } catch (Exception ex) {
+                //ignore
+            }
+        }
+    }
     public static Document read(XMLStreamReader reader) throws XMLStreamException {
         return read(reader, false);
     }
@@ -769,7 +869,8 @@ public final class StaxUtils {
     
     public static Document read(DocumentBuilder builder, XMLStreamReader reader, boolean repairing) 
         throws XMLStreamException {
-        Document doc = builder.newDocument();
+        
+        Document doc = builder == null ? DOMUtils.createDocument() : builder.newDocument();
         if (reader.getLocation().getSystemId() != null) {
             try {
                 doc.setDocumentURI(new String(reader.getLocation().getSystemId()));
@@ -789,80 +890,44 @@ public final class StaxUtils {
         return (parent instanceof Document) ? (Document)parent : parent.getOwnerDocument();
     }
 
-    /**
-     * @param parent
-     * @param reader
-     * @return
-     * @throws XMLStreamException
-     */
-    private static Element startElement(Document doc, 
-                                        Node parent, 
-                                        XMLStreamReader reader,
-                                        boolean repairing,
-                                        boolean recordLocation)
-        throws XMLStreamException {
-
-        Element e = doc.createElementNS(reader.getNamespaceURI(), reader.getLocalName());
-        if (reader.getPrefix() != null) {
-            e.setPrefix(reader.getPrefix());
-        }       
-        e = (Element)parent.appendChild(e);
-        recordLocation = addLocation(doc, e, reader, recordLocation);
-
-        for (int ns = 0; ns < reader.getNamespaceCount(); ns++) {
-            String uri = reader.getNamespaceURI(ns);
-            String prefix = reader.getNamespacePrefix(ns);
-
-            declare(e, uri, prefix);
-        }
-
-        for (int att = 0; att < reader.getAttributeCount(); att++) {
-            String name = reader.getAttributeLocalName(att);
-            String prefix = reader.getAttributePrefix(att);
-            if (prefix != null && prefix.length() > 0) {
-                name = prefix + ":" + name;
-            }
-
-            Attr attr = doc.createAttributeNS(reader.getAttributeNamespace(att), name);
-            attr.setValue(reader.getAttributeValue(att));
-            e.setAttributeNode(attr);
-        }
-
-        if (repairing && !isDeclared(e, reader.getNamespaceURI(), reader.getPrefix())) {
-            declare(e, reader.getNamespaceURI(), reader.getPrefix());
-        }
-
-        reader.next();
-
-        readDocElements(doc, e, reader, repairing, recordLocation);
-
-        return e;
-    }
-
     private static boolean isDeclared(Element e, String namespaceURI, String prefix) {
-        Attr att;
-        if (prefix != null && prefix.length() > 0) {
-            att = e.getAttributeNodeNS(XML_NS, prefix);
-        } else {
-            att = e.getAttributeNode("xmlns");
+        while (e != null) {
+            Attr att;
+            if (prefix != null && prefix.length() > 0) {
+                att = e.getAttributeNodeNS(XML_NS, prefix);
+            } else {
+                att = e.getAttributeNode("xmlns");
+            }
+    
+            if (att != null && att.getNodeValue().equals(namespaceURI)) {
+                return true;
+            }
+    
+            if (e.getParentNode() instanceof Element) {
+                e = (Element)e.getParentNode();
+            } else if (StringUtils.isEmpty(prefix) && StringUtils.isEmpty(namespaceURI)) {
+                //A document that probably doesn't have any namespace qualifies elements
+                return true;
+            } else {
+                e = null;
+            }
         }
-
-        if (att != null && att.getNodeValue().equals(namespaceURI)) {
-            return true;
-        }
-
-        if (e.getParentNode() instanceof Element) {
-            return isDeclared((Element)e.getParentNode(), namespaceURI, prefix);
-        }
-
         return false;
     }
+    
     public static void readDocElements(Node parent, XMLStreamReader reader, boolean repairing) 
         throws XMLStreamException {
         Document doc = getDocument(parent);
         readDocElements(doc, parent, reader, repairing, false);
     }
 
+    public static void readDocElements(Node parent, XMLStreamReader reader, boolean repairing, 
+                                       boolean isThreshold) 
+        throws XMLStreamException {
+        Document doc = getDocument(parent);
+        readDocElements(doc, parent, reader, repairing, false, isThreshold);
+    }
+    
     /**
      * @param parent
      * @param reader
@@ -871,19 +936,80 @@ public final class StaxUtils {
     public static void readDocElements(Document doc, Node parent,
                                        XMLStreamReader reader, boolean repairing, boolean recordLoc)
         throws XMLStreamException {
-
+        readDocElements(doc, parent, reader, repairing, recordLoc, false);
+    }
+    
+    /**
+     * @param parent
+     * @param reader
+     * @throws XMLStreamException
+     */
+    public static void readDocElements(Document doc, Node parent,
+                                       XMLStreamReader reader, boolean repairing, boolean recordLoc, 
+                                       boolean isThreshold)
+        throws XMLStreamException {
+        Stack<Node> stack = new Stack<Node>();
         int event = reader.getEventType();
+        int elementCount = 0;
         while (reader.hasNext()) {
             switch (event) {
-            case XMLStreamConstants.START_ELEMENT:
-                startElement(doc, parent, reader, repairing, recordLoc);
-                
+            case XMLStreamConstants.START_ELEMENT: {
+                elementCount++;
+                Element e;
+                if (!StringUtils.isEmpty(reader.getPrefix())) {
+                    e = doc.createElementNS(reader.getNamespaceURI(), 
+                                            reader.getPrefix() + ":" + reader.getLocalName());
+                } else {
+                    e = doc.createElementNS(reader.getNamespaceURI(), reader.getLocalName());
+                }
+                e = (Element)parent.appendChild(e);
+                recordLoc = addLocation(doc, e, reader, recordLoc);
+
+                for (int ns = 0; ns < reader.getNamespaceCount(); ns++) {
+                    String uri = reader.getNamespaceURI(ns);
+                    String prefix = reader.getNamespacePrefix(ns);
+
+                    declare(e, uri, prefix);
+                }
+
+                for (int att = 0; att < reader.getAttributeCount(); att++) {
+                    String name = reader.getAttributeLocalName(att);
+                    String prefix = reader.getAttributePrefix(att);
+                    if (prefix != null && prefix.length() > 0) {
+                        name = prefix + ":" + name;
+                    }
+
+                    Attr attr = doc.createAttributeNS(reader.getAttributeNamespace(att), name);
+                    attr.setValue(reader.getAttributeValue(att));
+                    e.setAttributeNode(attr);
+                }
+
+                if (repairing && !isDeclared(e, reader.getNamespaceURI(), reader.getPrefix())) {
+                    declare(e, reader.getNamespaceURI(), reader.getPrefix());
+                }
+                stack.push(parent);
+                if (isThreshold && innerElementLevelThreshold != -1 
+                    && stack.size() >= innerElementLevelThreshold) {
+                    throw new RuntimeException("reach the innerElementLevelThreshold:" 
+                                               + innerElementLevelThreshold);
+                }
+                if (isThreshold && innerElementCountThreshold != -1 
+                    && elementCount >= innerElementCountThreshold) {
+                    throw new RuntimeException("reach the innerElementCountThreshold:" 
+                                               + innerElementCountThreshold);
+                }
+                parent = e;
+                break;
+            }
+            case XMLStreamConstants.END_ELEMENT:
+                if (stack.isEmpty()) {
+                    return;
+                }
+                parent = stack.pop();
                 if (parent instanceof Document) {
                     return;
                 }
                 break;
-            case XMLStreamConstants.END_ELEMENT:
-                return;
             case XMLStreamConstants.NAMESPACE:
                 break;
             case XMLStreamConstants.ATTRIBUTE:
@@ -1083,6 +1209,10 @@ public final class StaxUtils {
                 } catch (Exception ex) {
                     //ignore
                 }
+            } else if (source instanceof StaxSource) {
+                return ((StaxSource)source).getXMLStreamReader();
+            } else if (source instanceof SAXSource) {
+                return createXMLStreamReader(((SAXSource)source).getInputSource());
             }
             
             XMLInputFactory factory = getXMLInputFactory();
@@ -1149,7 +1279,7 @@ public final class StaxUtils {
         String localName = value.substring(index + 1);
         String ns = reader.getNamespaceURI(prefix);
 
-        if ((prefix != null && ns == null) || localName == null) {
+        if ((!StringUtils.isEmpty(prefix) && ns == null) || localName == null) {
             throw new RuntimeException("Invalid QName in mapping: " + value);
         }
 
