@@ -170,12 +170,11 @@ public class ClientImpl
     /**
      * Create a Client that uses a specific EndpointImpl.
      * @param bus
-     * @param wsdlUrl
      * @param service
      * @param port
      * @param endpointImplFactory
      */
-    public ClientImpl(Bus bus, URL wsdlUrl, Service svc, QName port,
+    public ClientImpl(Bus bus, Service svc, QName port,
                       EndpointImplFactory endpointImplFactory) {
         this.bus = bus;
         outFaultObserver = new ClientOutFaultObserver(bus);
@@ -335,7 +334,6 @@ public class ClientImpl
                            Exchange exchange) throws Exception {
         Map<String, Object> context = new HashMap<String, Object>();
         Map<String, Object> resp = new HashMap<String, Object>();
-        resp.clear();
         Map<String, Object> req = new HashMap<String, Object>(getRequestContext());
         context.put(RESPONSE_CONTEXT, resp);
         context.put(REQUEST_CONTEXT, req);
