@@ -26,6 +26,8 @@ import java.lang.reflect.Method;
 import java.net.HttpURLConnection;
 import java.net.URI;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.LinkedHashMap;
@@ -35,12 +37,12 @@ import java.util.Map;
 import java.util.ResourceBundle;
 import java.util.logging.Logger;
 
-import javax.ws.rs.WebApplicationException;
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.MultivaluedMap;
 import javax.ws.rs.core.Response;
 import javax.ws.rs.core.UriBuilder;
+import javax.xml.stream.XMLStreamWriter;
 
 import org.apache.cxf.common.i18n.BundleUtils;
 import org.apache.cxf.common.logging.LogUtils;
@@ -70,15 +72,25 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
 
     private static final Logger LOG = LogUtils.getL7dLogger(ClientProxyImpl.class);
     private static final ResourceBundle BUNDLE = BundleUtils.getBundle(ClientProxyImpl.class);
+    private static final String SLASH = "/";
     
     private ClassResourceInfo cri;
     private boolean inheritHeaders;
     private boolean isRoot;
-    private Map<String, Object> valuesMap;
+    private Map<String, Object> valuesMap = Collections.emptyMap();
     
-    public ClientProxyImpl(URI baseURI, URI currentURI, ClassResourceInfo cri, boolean isRoot, 
+    public ClientProxyImpl(URI baseURI, ClassResourceInfo cri, boolean isRoot, 
                            boolean inheritHeaders, Object... varValues) {
-        super(baseURI, currentURI);
+        super(baseURI);
+        this.cri = cri;
+        this.isRoot = isRoot;
+        this.inheritHeaders = inheritHeaders;
+        initValuesMap(varValues);
+    }
+    
+    public ClientProxyImpl(ClientState initialState, ClassResourceInfo cri, boolean isRoot, 
+                           boolean inheritHeaders, Object... varValues) {
+        super(initialState);
         this.cri = cri;
         this.isRoot = isRoot;
         this.inheritHeaders = inheritHeaders;
@@ -86,21 +98,23 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
     }
     
     private void initValuesMap(Object... varValues) {
-        if (isRoot && varValues.length != 0) {
-            valuesMap = new LinkedHashMap<String, Object>();
+        if (isRoot) {
             List<String> vars = cri.getURITemplate().getVariables();
+            valuesMap = new LinkedHashMap<String, Object>();
             for (int i = 0; i < vars.size(); i++) {
-                if (i < varValues.length) {
-                    valuesMap.put(vars.get(i), varValues[i]);
+                if (varValues.length > 0) {
+                    if (i < varValues.length) {
+                        valuesMap.put(vars.get(i), varValues[i]);
+                    } else {
+                        org.apache.cxf.common.i18n.Message msg = new org.apache.cxf.common.i18n.Message(
+                             "ROOT_VARS_MISMATCH", BUNDLE, vars.size(), varValues.length);
+                        LOG.info(msg.toString());
+                        break;
+                    }
                 } else {
-                    org.apache.cxf.common.i18n.Message msg = new org.apache.cxf.common.i18n.Message(
-                         "ROOT_VARS_MISMATCH", BUNDLE, vars.size(), varValues.length);
-                    LOG.info(msg.toString());
-                    break;
+                    valuesMap.put(vars.get(i), "");
                 }
             }
-        } else {
-            valuesMap = Collections.emptyMap();
         }
     }
     
@@ -130,9 +144,10 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         
         UriBuilder builder = getCurrentBuilder().clone(); 
         if (isRoot) {
-            builder.path(ori.getClassResourceInfo().getURITemplate().getValue());
+            addNonEmptyPath(builder, ori.getClassResourceInfo().getURITemplate().getValue());
         }
-        builder.path(ori.getURITemplate().getValue());
+        addNonEmptyPath(builder, ori.getURITemplate().getValue());
+        
         handleMatrixes(types, params, builder);
         handleQueries(types, params, builder);
         
@@ -148,15 +163,17 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
             if (subCri == null) {
                 reportInvalidResourceMethod(m, "INVALID_SUBRESOURCE");
             }
-            ClientProxyImpl proxyImpl = new ClientProxyImpl(getBaseURI(), uri, subCri, false, inheritHeaders);
-            proxyImpl.setConfiguration(getConfiguration());
             
-            Object proxy = JAXRSClientFactory.create(m.getReturnType(), proxyImpl);
+            MultivaluedMap<String, String> subHeaders = paramHeaders;
             if (inheritHeaders) {
-                WebClient.client(proxy).headers(headers);
+                subHeaders.putAll(headers);    
             }
-            WebClient.client(proxy).headers(paramHeaders);
-            return proxy;
+            
+            ClientState newState = getState().newState(uri, subHeaders, 
+                 getTemplateParametersMap(ori.getURITemplate(), pathParams));
+            ClientProxyImpl proxyImpl = new ClientProxyImpl(newState, subCri, false, inheritHeaders);
+            proxyImpl.setConfiguration(getConfiguration());
+            return JAXRSClientFactory.create(m.getReturnType(), proxyImpl);
         } 
         
         headers.putAll(paramHeaders);
@@ -167,6 +184,12 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         
     }
 
+    private void addNonEmptyPath(UriBuilder builder, String pathValue) {
+        if (!SLASH.equals(pathValue)) {
+            builder.path(pathValue);
+        }
+    }
+    
     private static MultivaluedMap<ParameterType, Parameter> getParametersInfo(OperationResourceInfo ori) {
         MultivaluedMap<ParameterType, Parameter> map = 
             new MetadataMap<ParameterType, Parameter>();
@@ -177,7 +200,8 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         }
         for (Parameter p : parameters) {
             if (p.getType() == ParameterType.CONTEXT) {
-                reportInvalidResourceMethod(ori.getMethodToInvoke(), "NO_CONTEXT_PARAMETERS");
+                // ignore
+                continue;
             }
             map.add(p.getType(), p);
         }
@@ -219,7 +243,7 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
             } 
                         
             if (t == null) {
-                t = new WebApplicationException(r);
+                t = new ServerWebApplicationException(r);
             }
 
             
@@ -260,9 +284,7 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
             if (formParams || bodyClass != null && MultivaluedMap.class.isAssignableFrom(bodyClass)) {
                 headers.putSingle(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_FORM_URLENCODED);
             } else {
-                String cType = 
-                    bodyClass != null && InjectionUtils.isPrimitive(bodyClass) 
-                        ? MediaType.TEXT_PLAIN : ori.getConsumeTypes().isEmpty() 
+                String cType = ori.getConsumeTypes().isEmpty() 
                     || ori.getConsumeTypes().get(0).equals(MediaType.WILDCARD_TYPE) 
                     ? MediaType.APPLICATION_XML : ori.getConsumeTypes().get(0).toString();   
                 headers.putSingle(HttpHeaders.CONTENT_TYPE, cType);
@@ -271,11 +293,18 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         
         List<MediaType> accepts = getAccept(headers);
         if (accepts == null) {
-            accepts = InjectionUtils.isPrimitive(responseClass) 
-                ? Collections.singletonList(MediaType.TEXT_PLAIN_TYPE)
-                : ori.getProduceTypes().size() == 0 
-                || ori.getProduceTypes().get(0).equals(MediaType.WILDCARD_TYPE) 
-                ? Collections.singletonList(MediaType.APPLICATION_XML_TYPE) : ori.getProduceTypes();
+            boolean produceWildcard = ori.getProduceTypes().size() == 0 
+                || ori.getProduceTypes().get(0).equals(MediaType.WILDCARD_TYPE);
+            if (produceWildcard) {
+                accepts = InjectionUtils.isPrimitive(responseClass)
+                    ? Collections.singletonList(MediaType.TEXT_PLAIN_TYPE)
+                    : Collections.singletonList(MediaType.APPLICATION_XML_TYPE);        
+            } else if (responseClass == Void.class) {
+                accepts = Collections.singletonList(MediaType.WILDCARD_TYPE);
+            } else {
+                accepts = ori.getProduceTypes();
+            }
+            
             for (MediaType mt : accepts) {
                 headers.add(HttpHeaders.ACCEPT, mt.toString());
             }
@@ -299,40 +328,46 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
     private List<Object> getPathParamValues(MultivaluedMap<ParameterType, Parameter> map,
                                             Object[] params,
                                             OperationResourceInfo ori) {
-        List<Parameter> paramsList =  getParameters(map, ParameterType.PATH);
         List<Object> list = new LinkedList<Object>();
         if (isRoot) {
             list.addAll(valuesMap.values());
         }
-        List<String> vars = ori.getURITemplate().getVariables();
-        // TODO : unfortunately, UriBuilder will lose a method-scoped parameter 
-        // if a same name variable exists in a class scope which is an api bug.
-        // It's a rare case but we might want just to use UriBuilderImpl() directly 
-        // on the client side and tell it to choose the last variable value
+        List<String> methodVars = ori.getURITemplate().getVariables();
+        
+        List<Parameter> paramsList =  getParameters(map, ParameterType.PATH);
+        Map<String, Parameter> paramsMap = new LinkedHashMap<String, Parameter>();
         for (Parameter p : paramsList) {
-            if (valuesMap.containsKey(p.getName()) && !vars.contains(p.getName())) {
+            if (p.getName().length() == 0) {
+                MultivaluedMap<String, Object> values = 
+                    InjectionUtils.extractValuesFromBean(params[p.getIndex()], "");
+                for (String var : methodVars) {
+                    list.addAll(values.get(var));
+                }
+            } else {
+                paramsMap.put(p.getName(), p);
+            }
+        }
+        
+        for (String varName : methodVars) {
+            Parameter p = paramsMap.remove(varName);
+            if (p != null) {
+                list.add(params[p.getIndex()]);
+            }
+        }
+        
+        for (Parameter p : paramsMap.values()) {
+            if (valuesMap.containsKey(p.getName())) {
                 int index = 0; 
                 for (Iterator<String> it = valuesMap.keySet().iterator(); it.hasNext(); index++) {
-                    if (it.next().equals(p.getName())) {
+                    if (it.next().equals(p.getName()) && index < list.size()) {
                         list.remove(index);
                         list.add(index, params[p.getIndex()]);
                         break;
                     }
                 }
-            } else {
-                String paramName = p.getName();
-                if (!"".equals(paramName)) {
-                    list.add(params[p.getIndex()]);
-                } else {
-                    MultivaluedMap<String, Object> values = 
-                        InjectionUtils.extractValuesFromBean(params[p.getIndex()], "");
-                    for (String var : vars) {
-                        list.addAll(values.get(var));
-                    }
-                }
-                
-            }
+            }    
         }
+        
         return list;
     }
     
@@ -363,15 +398,25 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
         }
     }
 
-    private MultivaluedMap<String, Object> handleForm(MultivaluedMap<ParameterType, Parameter> map, 
+    private MultivaluedMap<String, String> handleForm(MultivaluedMap<ParameterType, Parameter> map, 
                                                       Object[] params) {
         
-        MultivaluedMap<String, Object> form = new MetadataMap<String, Object>();
+        MultivaluedMap<String, String> form = new MetadataMap<String, String>();
         
         List<Parameter> fm = getParameters(map, ParameterType.FORM);
         for (Parameter p : fm) {
-            if (params[p.getIndex()] != null) {
-                FormUtils.addPropertyToForm(form, p.getName(), params[p.getIndex()].toString());
+            Object pValue = params[p.getIndex()];
+            if (pValue != null) {
+                if (InjectionUtils.isSupportedCollectionOrArray(pValue.getClass())) {
+                    Collection<?> c = pValue.getClass().isArray() 
+                        ? Arrays.asList((Object[]) pValue) : (Collection) pValue;
+                    for (Iterator<?> it = c.iterator(); it.hasNext();) {
+                        FormUtils.addPropertyToForm(form, p.getName(), it.next());
+                    }
+                } else { 
+                    FormUtils.addPropertyToForm(form, p.getName(), pValue); 
+                }
+                
             }
         }
         
@@ -404,16 +449,10 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
                           List<Object> pathParams) throws Throwable {
         Message outMessage = createMessage(ori.getHttpMethod(), headers, uri);
         outMessage.getExchange().setOneWay(ori.isOneway());
-        if (pathParams.size() != 0) { 
-            List<String> vars = ori.getURITemplate().getVariables();
-            MultivaluedMap<String, String> templatesMap =  new MetadataMap<String, String>(vars.size());
-            for (int i = 0; i < vars.size(); i++) {
-                if (i < pathParams.size()) {
-                    templatesMap.add(vars.get(i), pathParams.get(i).toString());
-                }
-            }
-            outMessage.put(URITemplate.TEMPLATE_PARAMETERS, templatesMap);
-        }
+        
+        getState().setTemplates(getTemplateParametersMap(ori.getURITemplate(), pathParams));
+        outMessage.put(URITemplate.TEMPLATE_PARAMETERS, getState().getTemplates());
+        
         outMessage.setContent(OperationResourceInfo.class, ori);
         setPlainOperationNameProperty(outMessage, ori.getMethodToInvoke().getName());
         outMessage.getExchange().put(Method.class, ori.getMethodToInvoke());
@@ -443,20 +482,24 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
     
     protected Object handleResponse(HttpURLConnection connect, Message outMessage, OperationResourceInfo ori) 
         throws Throwable {
-        Response r = setResponseBuilder(connect, outMessage.getExchange()).clone().build();
-        Method method = ori.getMethodToInvoke();
-        checkResponse(method, r, outMessage);
-        if (method.getReturnType() == Void.class) { 
-            return null;
-        }
-        if (method.getReturnType() == Response.class
-            && (r.getEntity() == null || InputStream.class.isAssignableFrom(r.getEntity().getClass())
-                && ((InputStream)r.getEntity()).available() == 0)) {
-            return r;
-        }
+        try {
+            Response r = setResponseBuilder(connect, outMessage.getExchange()).build();
+            Method method = ori.getMethodToInvoke();
+            checkResponse(method, r, outMessage);
+            if (method.getReturnType() == Void.class) { 
+                return null;
+            }
+            if (method.getReturnType() == Response.class
+                && (r.getEntity() == null || InputStream.class.isAssignableFrom(r.getEntity().getClass())
+                    && ((InputStream)r.getEntity()).available() == 0)) {
+                return r;
+            }
         
-        return readBody(r, connect, outMessage, method.getReturnType(), 
+            return readBody(r, outMessage, method.getReturnType(), 
                         method.getGenericReturnType(), method.getDeclaredAnnotations());
+        } finally {
+            ProviderFactory.getInstance(outMessage).clearThreadLocalProxies();
+        }
     }
 
     public Object getInvocationHandler() {
@@ -470,7 +513,7 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
                                                    m.getDeclaringClass().getName(), 
                                                    m.getName());
         LOG.severe(errorMsg.toString());
-        throw new WebApplicationException(405);
+        throw new ClientWebApplicationException(errorMsg.toString());
     }
     
     // TODO : what we really need to do is to refactor JAXRSOutInterceptor so that
@@ -489,7 +532,8 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
             
             OperationResourceInfo ori = outMessage.getContent(OperationResourceInfo.class);
             OutputStream os = outMessage.getContent(OutputStream.class);
-            if (os == null || ori == null) {
+            if ((os == null && outMessage.getContent(XMLStreamWriter.class) == null)
+                || ori == null) {
                 return;
             }
             MessageContentsList objs = MessageContentsList.getContentsList(outMessage);
@@ -513,7 +557,6 @@ public class ClientProxyImpl extends AbstractClient implements InvocationHandler
                     writeBody(body, outMessage, body.getClass(), body.getClass(), 
                               anns, headers, os);
                 }
-                os.flush();
             } catch (Exception ex) {
                 throw new Fault(ex);
             }
