@@ -29,6 +29,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
+import java.util.Map;
 
 import javax.ws.rs.Consumes;
 import javax.ws.rs.Produces;
@@ -51,8 +52,10 @@ import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.jaxrs.Customer;
 import org.apache.cxf.jaxrs.CustomerParameterHandler;
 import org.apache.cxf.jaxrs.JAXBContextProvider;
+import org.apache.cxf.jaxrs.JAXBContextProvider2;
 import org.apache.cxf.jaxrs.ext.ParameterHandler;
 import org.apache.cxf.jaxrs.ext.RequestHandler;
+import org.apache.cxf.jaxrs.impl.MetadataMap;
 import org.apache.cxf.jaxrs.impl.WebApplicationExceptionMapper;
 import org.apache.cxf.jaxrs.model.ClassResourceInfo;
 import org.apache.cxf.jaxrs.model.ProviderInfo;
@@ -210,6 +213,16 @@ public class ProviderFactoryTest extends Assert {
     }
     
     @Test
+    public void testExceptionMappersHierarchyWithGenerics() throws Exception {
+        ProviderFactory pf = ProviderFactory.getInstance();
+        RuntimeExceptionMapper1 exMapper1 = new RuntimeExceptionMapper1(); 
+        pf.registerUserProvider(exMapper1);
+        RuntimeExceptionMapper2 exMapper2 = new RuntimeExceptionMapper2(); 
+        pf.registerUserProvider(exMapper2);
+        assertSame(exMapper1, pf.createExceptionMapper(RuntimeException.class, new MessageImpl()));
+        assertSame(exMapper2, pf.createExceptionMapper(WebApplicationException.class, new MessageImpl()));
+    }
+    
     public void testMessageBodyHandlerHierarchy() throws Exception {
         ProviderFactory pf = ProviderFactory.getInstance();
         List<Object> providers = new ArrayList<Object>();
@@ -232,6 +245,21 @@ public class ProviderFactoryTest extends Assert {
                                               MediaType.APPLICATION_XML_TYPE, new MessageImpl()));
     }
     
+    @Test
+    public void testMessageBodyWriterNoTypes() throws Exception {
+        ProviderFactory pf = ProviderFactory.getInstance();
+        List<Object> providers = new ArrayList<Object>();
+        SuperBookReaderWriter2 superBookHandler = new SuperBookReaderWriter2();
+        providers.add(superBookHandler);
+        pf.setUserProviders(providers);
+        assertSame(superBookHandler, 
+                   pf.createMessageBodyReader(SuperBook.class, SuperBook.class, new Annotation[]{}, 
+                                              MediaType.APPLICATION_XML_TYPE, new MessageImpl()));
+        assertSame(superBookHandler, 
+                   pf.createMessageBodyWriter(SuperBook.class, SuperBook.class, new Annotation[]{}, 
+                                              MediaType.APPLICATION_XML_TYPE, new MessageImpl()));
+    }
+    
     @Test
     public void testSortEntityProviders() throws Exception {
         ProviderFactory pf = ProviderFactory.getInstance();
@@ -306,10 +334,10 @@ public class ProviderFactoryTest extends Assert {
         
         MediaType mType = MediaType.valueOf(mediaType);
         
-        MessageBodyReader reader = pf.createMessageBodyReader(type, null, null, mType, new MessageImpl());
+        MessageBodyReader reader = pf.createMessageBodyReader(type, type, null, mType, new MessageImpl());
         assertSame("Unexpected provider found", provider, reader.getClass());
     
-        MessageBodyWriter writer = pf.createMessageBodyWriter(type, null, null, mType, new MessageImpl());
+        MessageBodyWriter writer = pf.createMessageBodyWriter(type, type, null, mType, new MessageImpl());
         assertTrue("Unexpected provider found", provider == writer.getClass());
     }
     
@@ -361,12 +389,79 @@ public class ProviderFactoryTest extends Assert {
     public void testRegisterCustomResolver() throws Exception {
         ProviderFactory pf = ProviderFactory.getInstance();
         pf.registerUserProvider(new JAXBContextProvider());
-        ContextResolver<JAXBContext> cr = pf.createContextResolver(JAXBContext.class, new MessageImpl());
+        Message message = prepareMessage("*/*", null);
+        ContextResolver<JAXBContext> cr = pf.createContextResolver(JAXBContext.class, message);
+        assertFalse(cr instanceof ProviderFactory.ContextResolverProxy);
         assertTrue("JAXBContext ContextProvider can not be found", 
                    cr instanceof JAXBContextProvider);
         
     }
     
+    @Test
+    public void testRegisterCustomResolver2() throws Exception {
+        ProviderFactory pf = ProviderFactory.getInstance();
+        pf.registerUserProvider(new JAXBContextProvider());
+        pf.registerUserProvider(new JAXBContextProvider2());
+        Message message = prepareMessage("text/xml+b", null);
+        ContextResolver<JAXBContext> cr = pf.createContextResolver(JAXBContext.class, message);
+        assertFalse(cr instanceof ProviderFactory.ContextResolverProxy);
+        assertTrue("JAXBContext ContextProvider can not be found", 
+                   cr instanceof JAXBContextProvider2);
+        
+    }
+    
+    @Test
+    public void testNoCustomResolver() throws Exception {
+        ProviderFactory pf = ProviderFactory.getInstance();
+        pf.registerUserProvider(new JAXBContextProvider());
+        pf.registerUserProvider(new JAXBContextProvider2());
+        Message message = prepareMessage("text/xml+c", null);
+        ContextResolver<JAXBContext> cr = pf.createContextResolver(JAXBContext.class, message);
+        assertNull(cr);
+    }
+    
+    @Test
+    public void testCustomResolverOut() throws Exception {
+        ProviderFactory pf = ProviderFactory.getInstance();
+        pf.registerUserProvider(new JAXBContextProvider());
+        pf.registerUserProvider(new JAXBContextProvider2());
+        Message message = prepareMessage("text/xml+c", "text/xml+a");
+        ContextResolver<JAXBContext> cr = pf.createContextResolver(JAXBContext.class, message);
+        assertFalse(cr instanceof ProviderFactory.ContextResolverProxy);
+        assertTrue("JAXBContext ContextProvider can not be found", 
+                   cr instanceof JAXBContextProvider);
+    }
+    
+    @Test
+    public void testCustomResolverProxy() throws Exception {
+        ProviderFactory pf = ProviderFactory.getInstance();
+        pf.registerUserProvider(new JAXBContextProvider());
+        pf.registerUserProvider(new JAXBContextProvider2());
+        Message message = prepareMessage("text/xml+*", null);
+        ContextResolver<JAXBContext> cr = pf.createContextResolver(JAXBContext.class, message);
+        assertTrue(cr instanceof ProviderFactory.ContextResolverProxy);
+        assertTrue(((ProviderFactory.ContextResolverProxy)cr).getResolvers().get(0) 
+                   instanceof JAXBContextProvider);
+        assertTrue(((ProviderFactory.ContextResolverProxy)cr).getResolvers().get(1) 
+                   instanceof JAXBContextProvider2);
+    }
+    
+    private Message prepareMessage(String contentType, String acceptType) {
+        Message message = new MessageImpl();
+        Map<String, List<String>> headers = new MetadataMap<String, String>();
+        message.put(Message.PROTOCOL_HEADERS, headers);
+        Exchange exchange = new ExchangeImpl();
+        exchange.setInMessage(message);
+        if (acceptType != null) {
+            headers.put("Accept", Collections.singletonList(acceptType));
+            exchange.setOutMessage(new MessageImpl());
+        } else {
+            headers.put("Content-Type", Collections.singletonList(contentType));
+        }
+        message.setExchange(exchange);
+        return message;
+    }
+    
     @Test
     public void testRegisterCustomEntityProvider() throws Exception {
         ProviderFactory pf = (ProviderFactory)ProviderFactory.getInstance();
@@ -602,6 +697,43 @@ public class ProviderFactoryTest extends Assert {
         
     }
     
+    @Produces("application/xml")
+    @Consumes("application/xml")
+    private static class SuperBookReaderWriter2 implements MessageBodyReader, MessageBodyWriter {
+
+        public boolean isReadable(Class type, Type genericType, Annotation[] annotations, 
+                                  MediaType mediaType) {
+            return true;
+        }
+
+        public Object readFrom(Class arg0, Type arg1, Annotation[] arg2, MediaType arg3, 
+                                  MultivaluedMap arg4, InputStream arg5) 
+            throws IOException, WebApplicationException {
+            // TODO Auto-generated method stub
+            return null;
+        }
+
+        public long getSize(Object t, Class type, Type genericType, 
+                            Annotation[] annotations, MediaType mediaType) {
+            // TODO Auto-generated method stub
+            return 0;
+        }
+
+        public boolean isWriteable(Class type, Type genericType, 
+                                   Annotation[] annotations, MediaType mediaType) {
+            return true;
+        }
+
+        
+        public void writeTo(Object arg0, Class arg1, Type arg2, 
+                            Annotation[] arg3, MediaType arg4, MultivaluedMap arg5, 
+                            OutputStream arg6) throws IOException, WebApplicationException {
+            // TODO Auto-generated method stub
+            
+        }
+        
+    }
+    
     private static class TestHandler implements RequestHandler {
 
         public Response handleRequest(Message m, ClassResourceInfo resourceClass) {
@@ -609,4 +741,24 @@ public class ProviderFactoryTest extends Assert {
         }
         
     }
+
+    private static class RuntimeExceptionMapper1 
+        extends AbstractTestExceptionMapper<RuntimeException> {
+        
+    }
+    
+    private static class RuntimeExceptionMapper2 
+        extends AbstractTestExceptionMapper<WebApplicationException> {
+        
+    }
+    
+    private static class AbstractTestExceptionMapper<T extends RuntimeException> 
+        implements ExceptionMapper<T> {
+
+        public Response toResponse(T arg0) {
+            // TODO Auto-generated method stub
+            return null;
+        }
+        
+    }
 }
