@@ -29,6 +29,7 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.TreeMap;
 
 import javax.servlet.ServletInputStream;
 import javax.servlet.ServletOutputStream;
@@ -697,8 +698,10 @@ public class JettyHTTPDestinationTest extends Assert {
                 //EasyMock.expect(request.getMethod()).andReturn(method);            
                 EasyMock.expect(request.getInputStream()).andReturn(is);
                 EasyMock.expect(request.getContextPath()).andReturn("/bar");
+                EasyMock.expect(request.getServletPath()).andReturn("");
                 EasyMock.expect(request.getPathInfo()).andReturn("/foo");
                 EasyMock.expect(request.getRequestURI()).andReturn("/foo");
+                EasyMock.expect(request.getRequestURL()).andReturn(new StringBuffer("http://test/request"));
                 EasyMock.expect(request.getCharacterEncoding()).andReturn("UTF-8");
                 EasyMock.expect(request.getQueryString()).andReturn(query);    
                 EasyMock.expect(request.getHeader("Accept")).andReturn("*/*");  
@@ -767,7 +770,7 @@ public class JettyHTTPDestinationTest extends Assert {
         outMsg.putAll(inMessage);
         outMsg.setExchange(new ExchangeImpl());
         outMsg.put(Message.PROTOCOL_HEADERS,
-                   new HashMap<String, List<String>>());
+                   new TreeMap<String, List<String>>(String.CASE_INSENSITIVE_ORDER));
         return outMsg;
     }
     
