@@ -23,6 +23,7 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.ResourceBundle;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.logging.Level;
@@ -64,6 +65,7 @@ import org.apache.cxf.ws.addressing.MAPAggregator;
 import org.apache.cxf.ws.addressing.Names;
 import org.apache.cxf.ws.addressing.ReferenceParametersType;
 import org.apache.cxf.ws.addressing.RelatesToType;
+import org.apache.cxf.ws.addressing.VersionTransformer.Names200408;
 import org.apache.cxf.wsdl.EndpointReferenceUtils;
 
 
@@ -75,6 +77,7 @@ public class MAPCodec extends AbstractSoapInterceptor {
 
     private static final Logger LOG = LogUtils.getL7dLogger(MAPCodec.class);
     private static final String IS_REFERENCE_PARAM_ATTR_NAME = "IsReferenceParameter";
+    private static final ResourceBundle BUNDLE = LOG.getResourceBundle();
     private static final String ONE_WAY_DECOUPLED_FAULT_SUPPORT = 
         "org.apache.cxf.ws.addressing.oneway.decoupled_fault_support";
     
@@ -244,21 +247,23 @@ public class MAPCodec extends AbstractSoapInterceptor {
                                     hdr, 
                                     marshaller);
                 }
-                encodeAsExposed(maps,
-                                message,
-                                maps.getReplyTo(), 
-                                Names.WSA_REPLYTO_QNAME, 
-                                EndpointReferenceType.class,
-                                hdr,
-                                marshaller);
-                if (Names.WSA_REPLYTO_QNAME.equals(duplicate)) {
+                if (needsReplyTo(maps)) {
                     encodeAsExposed(maps,
-                                    message,
-                                    maps.getReplyTo(), 
-                                    Names.WSA_REPLYTO_QNAME, 
-                                    EndpointReferenceType.class,
-                                    hdr,
-                                    marshaller);
+                            message,
+                            maps.getReplyTo(), 
+                            Names.WSA_REPLYTO_QNAME, 
+                            EndpointReferenceType.class,
+                            hdr,
+                            marshaller);
+                    if (Names.WSA_REPLYTO_QNAME.equals(duplicate)) {
+                        encodeAsExposed(maps,
+                                        message,
+                                        maps.getReplyTo(), 
+                                        Names.WSA_REPLYTO_QNAME, 
+                                        EndpointReferenceType.class,
+                                        hdr,
+                                        marshaller);
+                    }
                 }
 
                 encodeAsExposed(maps,
@@ -293,11 +298,7 @@ public class MAPCodec extends AbstractSoapInterceptor {
                                     hdr, 
                                     marshaller);
                 }
-                if (maps.getFaultTo() != null
-                    && maps.getFaultTo().getAddress() != null
-                    && maps.getFaultTo().getAddress().getValue() != null
-                    && !maps.getFaultTo().getAddress().getValue()
-                        .equals(maps.getReplyTo().getAddress().getValue())) {
+                if (needsFaultTo(maps)) {
                     encodeAsExposed(maps,
                                     message,
                                     maps.getFaultTo(), 
@@ -337,6 +338,23 @@ public class MAPCodec extends AbstractSoapInterceptor {
         }
     }
 
+    private boolean needsReplyTo(AddressingProperties maps) {
+        return maps.getReplyTo() != null 
+            && maps.getReplyTo().getAddress() != null
+            && maps.getReplyTo().getAddress().getValue() != null
+            && !(VersionTransformer.Names200408.WSA_NAMESPACE_NAME.equals(maps.getNamespaceURI()) 
+                && maps.getReplyTo().getAddress().getValue()
+                .equals(ContextUtils.getNoneEndpointReference().getAddress().getValue()));
+    }
+
+    private boolean needsFaultTo(AddressingProperties maps) {
+        return maps.getFaultTo() != null
+            && maps.getFaultTo().getAddress() != null
+            && maps.getFaultTo().getAddress().getValue() != null
+            && !maps.getFaultTo().getAddress().getValue()
+                .equals(maps.getReplyTo().getAddress().getValue());
+    }
+
     private void encodeReferenceParameters(AddressingProperties maps, Element header, 
                                            Marshaller marshaller) throws JAXBException {
         EndpointReferenceType toEpr = maps.getToEndpointReference();
@@ -348,15 +366,13 @@ public class MAPCodec extends AbstractSoapInterceptor {
                         JAXBElement jaxbEl = null;
                         if (o instanceof Element) {
                             Element e = (Element)o;
-                            QName elQn = new QName(e.getNamespaceURI(), e.getLocalName());
-                            jaxbEl = new JAXBElement<String>(elQn,
-                                String.class, 
-                                e.getTextContent());
+                            Node importedNode = header.getOwnerDocument().importNode(e, true);
+                            header.appendChild(importedNode);                            
                         } else {
                             jaxbEl = (JAXBElement) o;
+                            marshaller.marshal(jaxbEl, header);
                         }
-                        marshaller.marshal(jaxbEl, header);
-                        
+                                               
                         Element lastAdded = (Element)header.getLastChild();
                         addIsReferenceParameterMarkerAttribute(lastAdded, maps.getNamespaceURI());
                     } else {
@@ -479,7 +495,7 @@ public class MAPCodec extends AbstractSoapInterceptor {
                 LOG.log(Level.FINE, "Inbound WS-Addressing headers");
                 Unmarshaller unmarshaller = null;
                 Set<Element> referenceParameterHeaders = null;
-
+                QName invalidCardinalityQName = null;
                 Iterator<Header> iter = header.iterator();
                 while (iter.hasNext()) {
                     Header hdr = iter.next();
@@ -502,12 +518,15 @@ public class MAPCodec extends AbstractSoapInterceptor {
                             }
                             String localName = headerElement.getLocalName();
                             if (Names.WSA_MESSAGEID_NAME.equals(localName)) {
+                                invalidCardinalityQName = maps.getMessageID() != null
+                                    ? Names.WSA_MESSAGEID_QNAME : null;
                                 maps.setMessageID(decodeAsNative(
                                                        headerURI,
                                                        AttributedURIType.class,
                                                        headerElement, 
                                                        unmarshaller));
                             } else if (Names.WSA_TO_NAME.equals(localName)) {
+                                invalidCardinalityQName = maps.getTo() != null ? Names.WSA_TO_QNAME : null;
                                 AttributedURIType addr = decodeAsNative(
                                                        headerURI,
                                                        AttributedURIType.class,
@@ -515,18 +534,26 @@ public class MAPCodec extends AbstractSoapInterceptor {
                                                        unmarshaller);
                                 maps.setTo(EndpointReferenceUtils.getEndpointReference(addr));
                             } else if (Names.WSA_FROM_NAME.equals(localName)) {
+                                invalidCardinalityQName = maps.getFrom() != null
+                                    ? Names.WSA_FROM_QNAME : null;
                                 maps.setFrom(decodeAsNative(
                                                        headerURI,
                                                        EndpointReferenceType.class,
                                                        headerElement, 
                                                        unmarshaller));
                             } else if (Names.WSA_REPLYTO_NAME.equals(localName)) {
+                                invalidCardinalityQName = maps.getReplyTo() != null
+                                                                   ? Names.WSA_REPLYTO_QNAME : null;
+
                                 maps.setReplyTo(decodeAsNative(
                                                        headerURI,
                                                        EndpointReferenceType.class,
                                                        headerElement, 
                                                        unmarshaller));
                             } else if (Names.WSA_FAULTTO_NAME.equals(localName)) {
+                                invalidCardinalityQName = maps.getFaultTo() != null
+                                    ? Names.WSA_FAULTTO_QNAME : null;
+
                                 maps.setFaultTo(decodeAsNative(
                                                        headerURI,
                                                        EndpointReferenceType.class,
@@ -539,6 +566,8 @@ public class MAPCodec extends AbstractSoapInterceptor {
                                                        headerElement, 
                                                        unmarshaller));
                             } else if (Names.WSA_ACTION_NAME.equals(localName)) {
+                                invalidCardinalityQName = maps.getAction() != null
+                                    ? Names.WSA_ACTION_QNAME : null;
                                 maps.setAction(decodeAsNative(
                                                        headerURI,
                                                        AttributedURIType.class,
@@ -560,13 +589,24 @@ public class MAPCodec extends AbstractSoapInterceptor {
                 if (null != referenceParameterHeaders && null != maps) {
                     decodeReferenceParameters(referenceParameterHeaders, maps, unmarshaller);
                 }
+                if (invalidCardinalityQName != null) {
+                    storeInvalidCardinalityFault(message, invalidCardinalityQName);
+                }
+
             }
         } catch (JAXBException je) {
             LOG.log(Level.WARNING, "SOAP_HEADER_DECODE_FAILURE_MSG", je); 
         }
         return maps;
     }
-    
+        
+    private void storeInvalidCardinalityFault(SoapMessage message, QName wsaHeaderName) {
+        LOG.log(Level.WARNING, "INVALID_CARDINALITY_MESSAGE", wsaHeaderName);
+        String reason = BUNDLE.getString("INVALID_ADDRESSING_PROPERTY_MESSAGE");
+        ContextUtils.storeMAPFaultName(Names.INVALID_CARDINALITY_NAME, message);
+        ContextUtils.storeMAPFaultReason(reason, message);
+    }
+        
     private void decodeReferenceParameters(Set<Element> referenceParameterHeaders, 
                                            AddressingPropertiesImpl maps, 
                                            Unmarshaller unmarshaller) 
@@ -574,8 +614,12 @@ public class MAPCodec extends AbstractSoapInterceptor {
         EndpointReferenceType toEpr = maps.getToEndpointReference();
         if (null != toEpr) {
             for (Element e : referenceParameterHeaders) {
-                JAXBElement<String> el = unmarshaller.unmarshal(e, String.class);
-                ContextUtils.applyReferenceParam(toEpr, el);
+                if (DOMUtils.getChild(e, Node.ELEMENT_NODE) == null) {
+                    JAXBElement<String> el = unmarshaller.unmarshal(e, String.class);
+                    ContextUtils.applyReferenceParam(toEpr, el);
+                } else {
+                    ContextUtils.applyReferenceParam(toEpr, e);
+                }
             }
         }
     }
@@ -687,7 +731,8 @@ public class MAPCodec extends AbstractSoapInterceptor {
         Iterator<Header> iter = header.iterator();
         while (iter.hasNext()) {
             Header hdr = iter.next();
-            if (Names.WSA_NAMESPACE_NAME.equals(hdr.getName().getNamespaceURI())) {
+            if (Names.WSA_NAMESPACE_NAME.equals(hdr.getName().getNamespaceURI())
+                || Names200408.WSA_NAMESPACE_NAME.equals(hdr.getName().getNamespaceURI())) {
                 iter.remove();
             }
         }
@@ -773,6 +818,7 @@ public class MAPCodec extends AbstractSoapInterceptor {
     private void restoreExchange(SoapMessage message, AddressingProperties maps) {
         if (maps != null
             && maps.getRelatesTo() != null
+            && !Names.WSA_UNSPECIFIED_RELATIONSHIP.equals(maps.getRelatesTo().getValue())
             && isRelationshipReply(maps.getRelatesTo())) { 
             Exchange correlatedExchange =
                 uncorrelatedExchanges.remove(maps.getRelatesTo().getValue());
@@ -780,7 +826,7 @@ public class MAPCodec extends AbstractSoapInterceptor {
                 synchronized (correlatedExchange) {
                     message.setExchange(correlatedExchange);
                 }
-            } else {
+            } else if (ContextUtils.isRequestor(message) && !message.getExchange().isOneWay()) {
                 if (ContextUtils.retrieveDeferUncorrelatedMessageAbort(message)) {
                     LOG.fine("deferring uncorrelated message abort");
                     ContextUtils.storeDeferredUncorrelatedMessageAbort(message);
@@ -819,7 +865,10 @@ public class MAPCodec extends AbstractSoapInterceptor {
      * @param message the current message
      */
     private void markPartialResponse(SoapMessage message, AddressingProperties maps) {
-        if (ContextUtils.isRequestor(message) && null != maps && null == maps.getRelatesTo()) {
+        if (ContextUtils.isRequestor(message) && null != maps
+            && (null == maps.getRelatesTo() 
+                || (null != maps.getRelatesTo()
+                    && Names.WSA_UNSPECIFIED_RELATIONSHIP.equals(maps.getRelatesTo().getValue())))) {
             message.put(Message.PARTIAL_RESPONSE_MESSAGE, Boolean.TRUE);
         } 
     }
