@@ -454,7 +454,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         String keyTypeTemplate = null;
         String sptt = null;
         
-        if (template != null) {
+        if (template != null && DOMUtils.getFirstElement(template) != null) {
             if (this.useSecondaryParameters()) {
                 writer.writeStartElement("wst", "SecondaryParameters", namespace);
             }
@@ -484,10 +484,12 @@ public class STSClient implements Configurable, InterceptorProvider {
         if (sptt == null) {
             addTokenType(writer);
         }
+        if (isSecureConv) {
+            addLifetime(writer);
+        }
         if (keyTypeTemplate == null) {
-            keyTypeTemplate = keyType;
+            keyTypeTemplate = writeKeyType(writer, keyType);
         }
-        keyTypeTemplate = writeKeyType(writer, keyTypeTemplate);
 
         byte[] requestorEntropy = null;
         X509Certificate cert = null;
@@ -792,7 +794,6 @@ public class STSClient implements Configurable, InterceptorProvider {
     private String writeKeyType(W3CDOMStreamWriter writer, String keyTypeToWrite) 
         throws XMLStreamException {
         if (isSecureConv) {
-            addLifetime(writer);
             if (keyTypeToWrite == null) {
                 writer.writeStartElement("wst", "TokenType", namespace);
                 writer.writeCharacters(STSUtils.getTokenTypeSCT(namespace));
@@ -837,7 +838,7 @@ public class STSClient implements Configurable, InterceptorProvider {
     private void addLifetime(XMLStreamWriter writer) throws XMLStreamException {
         Date creationTime = new Date();
         Date expirationTime = new Date();
-        expirationTime.setTime(creationTime.getTime() + (ttl * 1000));
+        expirationTime.setTime(creationTime.getTime() + (ttl * 1000L));
 
         XmlSchemaDateFormat fmt = new XmlSchemaDateFormat();
         writer.writeStartElement("wst", "Lifetime", namespace);
@@ -920,6 +921,7 @@ public class STSClient implements Configurable, InterceptorProvider {
         Element lte = null;
         Element entropy = null;
         String tt = null;
+        String retKeySize = null;
 
         while (el != null) {
             String ln = el.getLocalName();
@@ -938,6 +940,8 @@ public class STSClient implements Configurable, InterceptorProvider {
                     entropy = el;
                 } else if ("TokenType".equals(ln)) {
                     tt = DOMUtils.getContent(el);
+                } else if ("KeySize".equals(ln)) {
+                    retKeySize = DOMUtils.getContent(el);
                 }
             }
             el = DOMUtils.getNextElement(el);
@@ -962,32 +966,39 @@ public class STSClient implements Configurable, InterceptorProvider {
                 // First check for the binary secret
                 String b64Secret = DOMUtils.getContent(child);
                 secret = Base64.decode(b64Secret);
-            } else if (childQname.equals(new QName(namespace, WSConstants.ENC_KEY_LN))) {
-                try {
-
-                    EncryptedKeyProcessor processor = new EncryptedKeyProcessor();
-
-                    processor.handleToken(child, null, createCrypto(true), createHandler(), null,
-                                          new Vector(), null);
-
-                    secret = processor.getDecryptedBytes();
-                } catch (IOException e) {
-                    throw new TrustException("ENCRYPTED_KEY_ERROR", LOG, e);
-                }
+            } else if (childQname.equals(new QName(WSConstants.ENC_NS, WSConstants.ENC_KEY_LN))) {
+                secret = decryptKey(child);
             } else if (childQname.equals(new QName(namespace, "ComputedKey"))) {
                 // Handle the computed key
-                Element binSecElem = entropy == null ? null : DOMUtils.getFirstElement(entropy);
-                String content = binSecElem == null ? null : DOMUtils.getContent(binSecElem);
-                if (content != null && !StringUtils.isEmpty(content.trim())) {
-
-                    byte[] serviceEntr = Base64.decode(content);
+                Element computedKeyChild = entropy == null ? null : DOMUtils.getFirstElement(entropy);
+                byte[] serviceEntr = null;
 
+                if (computedKeyChild != null) {
+                    QName computedKeyChildQName = DOMUtils.getElementQName(computedKeyChild);
+                    if (computedKeyChildQName.equals(new QName(WSConstants.ENC_NS, WSConstants.ENC_KEY_LN))) {
+                        serviceEntr = decryptKey(computedKeyChild);
+                    } else if (computedKeyChildQName.equals(new QName(namespace, "BinarySecret"))) {
+                        String content = DOMUtils.getContent(computedKeyChild);
+                        serviceEntr = Base64.decode(content);
+                    }
+                }
+                
+                if (serviceEntr != null) {
                     // Right now we only use PSHA1 as the computed key algo
                     P_SHA1 psha1 = new P_SHA1();
 
-                    int length = (keySize > 0) ? keySize : 256;
-                    if (algorithmSuite != null) {
-                        length = (keySize > 0) ? keySize : algorithmSuite.getMaximumSymmetricKeyLength();
+                    int length = 0;
+                    if (retKeySize != null) {
+                        try {
+                            length = Integer.parseInt(retKeySize);
+                        } catch (NumberFormatException ex) {
+                            // do nothing
+                        }
+                    } else {
+                        length = keySize;
+                    }
+                    if (length <= 0) {
+                        length = 256;
                     }
                     try {
                         secret = psha1.createKey(requestorEntropy, serviceEntr, 0, length / 8);
@@ -1007,6 +1018,19 @@ public class STSClient implements Configurable, InterceptorProvider {
 
         return token;
     }
+    
+    private byte[] decryptKey(Element child) throws TrustException, WSSecurityException {
+        try {
+            EncryptedKeyProcessor processor = new EncryptedKeyProcessor();
+
+            processor.handleToken(child, null, createCrypto(true), createHandler(), null,
+                                  new Vector(), null);
+
+            return processor.getDecryptedBytes();
+        } catch (IOException e) {
+            throw new TrustException("ENCRYPTED_KEY_ERROR", LOG, e);
+        }
+    }
 
     private CallbackHandler createHandler() {
         Object o = getProperty(SecurityConstants.CALLBACK_HANDLER);
@@ -1058,7 +1082,7 @@ public class STSClient implements Configurable, InterceptorProvider {
                 properties.load(ins);
                 ins.close();
             } else {
-                throw new Fault("Could not find properties file " + url, LOG);
+                throw new Fault("Could not find properties file " + (String)o, LOG);
             }
         } else if (o instanceof URL) {
             properties = new Properties();
