@@ -21,8 +21,9 @@ package org.apache.cxf.systest.jaxrs;
 
 import java.io.File;
 import java.io.InputStream;
+import java.net.HttpURLConnection;
 import java.net.URL;
-import java.net.URLConnection;
+import java.net.URLEncoder;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -32,6 +33,8 @@ import java.util.Map;
 
 import javax.ws.rs.core.MultivaluedMap;
 import javax.ws.rs.core.Response;
+import javax.xml.bind.JAXBElement;
+import javax.xml.namespace.QName;
 
 import org.apache.commons.httpclient.Header;
 import org.apache.commons.httpclient.HttpClient;
@@ -45,11 +48,14 @@ import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.io.CachedOutputStream;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactoryBean;
+import org.apache.cxf.jaxrs.client.ResponseReader;
 import org.apache.cxf.jaxrs.client.ServerWebApplicationException;
 import org.apache.cxf.jaxrs.client.WebClient;
 import org.apache.cxf.jaxrs.ext.xml.XMLSource;
 import org.apache.cxf.jaxrs.provider.JAXBElementProvider;
 import org.apache.cxf.jaxrs.provider.XSLTJaxbProvider;
+import org.apache.cxf.systest.jaxrs.BookStore.BookInfo;
+import org.apache.cxf.systest.jaxrs.BookStore.BookInfoInterface;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
 
 import org.junit.BeforeClass;
@@ -65,6 +71,82 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                    launchServer(BookServer.class));
     }
     
+    @Test
+    public void testMalformedAcceptType() {
+        WebClient wc = 
+            WebClient.create("http://localhost:" + PORT + "/bookstore/books/123");
+        wc.accept("application");
+        Response r = wc.get();
+        assertEquals(406, r.getStatus());
+    }
+    
+    public void testGetBookWithCustomHeader() throws Exception {
+        
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/books/123"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        Book b = wc.get(Book.class);
+        assertEquals(123L, b.getId());
+        
+        MultivaluedMap<String, Object> headers = wc.getResponse().getMetadata();
+        assertEquals("123", headers.getFirst("BookId"));
+        assertEquals(MultivaluedMap.class.getName(), headers.getFirst("MAP-NAME"));
+        
+        assertNotNull(headers.getFirst("Date"));
+        
+        wc.header("PLAIN-MAP", "true");
+        b = wc.get(Book.class);
+        assertEquals(123L, b.getId());
+        
+        headers = wc.getResponse().getMetadata();
+        assertEquals("321", headers.getFirst("BookId"));
+        assertEquals(Map.class.getName(), headers.getFirst("MAP-NAME"));
+        
+        assertNotNull(headers.getFirst("Date"));
+    }
+    
+    @Test
+    public void testGetBookWithNameInQuery() throws Exception {
+        
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/name-in-query"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        String name = "Many        spaces";
+        wc.query("name", name);
+        Book b = wc.get(Book.class);
+        assertEquals(name, b.getName());
+    }
+    
+    @Test
+    public void testGetBookWithColonMarks() throws Exception {
+        
+        // URLEncoder will turn ":" into "%3A" but ':' is actually
+        // not disallowed in the path components 
+        String endpointAddressUrlEncoded =
+            "http://localhost:" + PORT + "/bookstore/books/colon/" 
+            + URLEncoder.encode("1:2:3", "UTF-8");
+        
+        Response r = WebClient.create(endpointAddressUrlEncoded).get();
+        assertEquals(404, r.getStatus());
+        
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/books/colon/1:2:3";
+        WebClient wc = WebClient.create(endpointAddress);
+        Book b = wc.get(Book.class);
+        assertEquals(123L, b.getId());
+    }
+    
+    @Test
+    public void testPostAnd401WithText() throws Exception {
+        
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/post401"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        WebClient.getConfig(wc).getHttpConduit().getClient().setAllowChunking(false);
+        Response r = wc.post(null);
+        assertEquals(401, r.getStatus());
+        assertEquals("This is 401", getStringFromInputStream((InputStream)r.getEntity()));
+    }
 
     @Test
     public void testGetCollectionOfBooks() throws Exception {
@@ -79,6 +161,24 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(123L, book.getId());
     }
     
+    @Test
+    public void testGetBookResponseAndETag() throws Exception {
+        
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/books/response/123"; 
+        WebClient wc = WebClient.create(endpointAddress);
+        Book book = wc.get(Book.class);
+        assertEquals(200, wc.getResponse().getStatus());
+        assertEquals(123L, book.getId());
+        MultivaluedMap<String, Object> headers = wc.getResponse().getMetadata();
+        assertTrue(headers.size() > 0);
+        Object etag = headers.getFirst("ETag");
+        assertNotNull(etag);
+        assertTrue(etag.toString().startsWith("\""));
+        assertTrue(etag.toString().endsWith("\""));
+
+    }
+    
     
     @Test
     public void testOnewayWebClient() throws Exception {
@@ -87,6 +187,20 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(202, r.getStatus());
     }
     
+    @Test
+    public void testBookWithSpace() throws Exception {
+        WebClient client = WebClient.create("http://localhost:" + PORT + "/bookstore/").path("the books/123");
+        Book book = client.get(Book.class);
+        assertEquals(123L, book.getId());
+    }
+    
+    @Test
+    public void testBookWithSpaceProxy() throws Exception {
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
+        Book book = store.getBookWithSpace("123");
+        assertEquals(123L, book.getId());
+    }
+    
     @Test
     public void testTempRedirectWebClient() throws Exception {
         WebClient client = WebClient.create("http://localhost:" + PORT + "/bookstore/tempredirect");
@@ -111,13 +225,13 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     }
     
     @Test
-    public void testSetTwoCookieWebClient() throws Exception {
-        WebClient client = WebClient.create("http://localhost:" + PORT + "/bookstore/settwocookies");
+    public void testSetManyCookiesWebClient() throws Exception {
+        WebClient client = WebClient.create("http://localhost:" + PORT + "/bookstore/setmanycookies");
         Response r = client.type("*/*").get();
         assertEquals(200, r.getStatus());
         List<Object> cookies = r.getMetadata().get("Set-Cookie");
         assertNotNull(cookies);
-        assertEquals(2, cookies.size());
+        assertEquals(3, cookies.size());
     }
 
     
@@ -140,7 +254,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     
     @Test
     public void testPropogateException() throws Exception {
-        getAndCompare("http://localhost:" + PORT + "/bookstore/propogateexception",
+        getAndCompare("http://localhost:" + PORT + "/bookstore/propagate-exception",
                       "", "application/xml", 500);
     }
     
@@ -150,14 +264,14 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
             + "<ns1:faultstring xmlns:ns1=\"http://cxf.apache.org/bindings/xformat\">"
             + "org.apache.cxf.systest.jaxrs.BookNotFoundFault: Book Exception</ns1:faultstring>"
             + "</ns1:XMLFault>";
-        getAndCompare("http://localhost:" + PORT + "/bookstore/propogateexception2",
+        getAndCompare("http://localhost:" + PORT + "/bookstore/propagate-exception2",
                       data, "application/xml", 500);
     }
     
     @Test
     public void testPropogateException3() throws Exception {
         String data = "<nobook/>";
-        getAndCompare("http://localhost:" + PORT + "/bookstore/propogateexception3",
+        getAndCompare("http://localhost:" + PORT + "/bookstore/propagate-exception3",
                       data, "application/xml", 500);
     }
     
@@ -182,6 +296,41 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         }
     }
     
+    @Test
+    public void testServerWebApplicationExceptionXML() throws Exception {
+        ResponseReader reader = new ResponseReader();
+        reader.setEntityClass(Book.class);
+        WebClient wc = WebClient.create("http://localhost:" + PORT + "/bookstore/webappexceptionXML",
+                                        Collections.singletonList(reader));
+        wc.accept("application/xml");
+        try {
+            wc.get(Book.class);
+            fail("Exception expected");
+        } catch (ServerWebApplicationException ex) {
+            assertEquals(406, ex.getStatus());
+            Book exBook = ex.toErrorObject(wc, Book.class);
+            assertEquals("Exception", exBook.getName());
+            assertEquals(999L, exBook.getId());
+        }
+    }
+    
+    @Test
+    public void testServerWebApplicationExceptionXMLWithProxy() throws Exception {
+        ResponseReader reader = new ResponseReader();
+        reader.setEntityClass(Book.class);
+        BookStore proxy = JAXRSClientFactory.create("http://localhost:" + PORT,
+                                                    BookStore.class, Collections.singletonList(reader));
+        try {
+            proxy.throwExceptionXML();
+            fail("Exception expected");
+        } catch (ServerWebApplicationException ex) {
+            assertEquals(406, ex.getStatus());
+            Book exBook = ex.toErrorObject(WebClient.client(proxy), Book.class);
+            assertEquals("Exception", exBook.getName());
+            assertEquals(999L, exBook.getId());
+        }
+    }
+    
     @Test
     public void testServerWebApplicationExceptionWithProxy() throws Exception {
         BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
@@ -236,6 +385,32 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals("CXF Rocks", b22.getName());
     }
     
+    @Test 
+    public void testGetBookCollection2() throws Exception {
+        JAXBElementProvider provider = new JAXBElementProvider();
+        provider.setMarshallAsJaxbElement(true);
+        provider.setUnmarshallAsJaxbElement(true);
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT,
+                                                    BookStore.class,
+                                                    Collections.singletonList(provider));
+        BookNoXmlRootElement b1 = new BookNoXmlRootElement("CXF in Action", 123L);
+        BookNoXmlRootElement b2 = new BookNoXmlRootElement("CXF Rocks", 124L);
+        List<BookNoXmlRootElement> books = new ArrayList<BookNoXmlRootElement>();
+        books.add(b1);
+        books.add(b2);
+        WebClient.getConfig(store).getHttpConduit().getClient().setReceiveTimeout(10000000L);
+        List<BookNoXmlRootElement> books2 = store.getBookCollection2(books);
+        assertNotNull(books2);
+        assertNotSame(books, books2);
+        assertEquals(2, books2.size());
+        BookNoXmlRootElement b11 = books.get(0);
+        assertEquals(123L, b11.getId());
+        assertEquals("CXF in Action", b11.getName());
+        BookNoXmlRootElement b22 = books.get(1);
+        assertEquals(124L, b22.getId());
+        assertEquals("CXF Rocks", b22.getName());
+    }
+    
     @Test 
     public void testGetBookArray() throws Exception {
         BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
@@ -318,6 +493,38 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(1, date.size());
     }
     
+    @Test
+    public void testExplicitOptions() throws Exception {
+        WebClient wc = 
+            WebClient.create("http://localhost:" 
+                             + PORT + "/bookstore/options");
+        Response response = wc.options();
+        List<Object> values = response.getMetadata().get("Allow");
+        assertNotNull(values);
+        assertTrue(values.contains("POST") && values.contains("GET")
+                   && values.contains("DELETE") && values.contains("PUT"));
+        assertEquals(0, ((InputStream)response.getEntity()).available());
+        List<Object> date = response.getMetadata().get("Date");
+        assertNotNull(date);
+        assertEquals(1, date.size());
+    }
+    
+    @Test
+    public void testOptionsOnSubresource() throws Exception {
+        WebClient wc = 
+            WebClient.create("http://localhost:" 
+                             + PORT + "/bookstore/booksubresource/123");
+        Response response = wc.options();
+        List<Object> values = response.getMetadata().get("Allow");
+        assertNotNull(values);
+        assertTrue(!values.contains("POST") && values.contains("GET")
+                   && !values.contains("DELETE") && values.contains("PUT"));
+        assertEquals(0, ((InputStream)response.getEntity()).available());
+        List<Object> date = response.getMetadata().get("Date");
+        assertNotNull(date);
+        assertEquals(1, date.size());
+    }
+    
     @Test
     public void testEmptyPost() throws Exception {
         WebClient wc = 
@@ -327,16 +534,40 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(204, response.getStatus());
     }
     
+    @Test
+    public void testEmptyPut() throws Exception {
+        WebClient wc = 
+            WebClient.create("http://localhost:" 
+                             + PORT + "/bookstore/emptyput");
+        Response response = wc.put(null);
+        assertEquals(204, response.getStatus());
+        
+        response = wc.put("");
+        assertEquals(204, response.getStatus());
+    }
+    
     @Test
     public void testEmptyPostProxy() throws Exception {
+        String address = "http://localhost:" + PORT;
         JAXRSClientFactoryBean bean = new JAXRSClientFactoryBean(); 
-        bean.setAddress("http://localhost:" + PORT);
+        bean.setAddress(address);
         bean.setResourceClass(BookStore.class);
         BookStore store = bean.create(BookStore.class);
         store.emptypost();
         assertEquals(204, WebClient.client(store).getResponse().getStatus());
     }
     
+    @Test
+    public void testEmptyPostProxy2() throws Exception {
+        String address = "http://localhost:" + PORT;
+        JAXRSClientFactoryBean bean = new JAXRSClientFactoryBean();
+        bean.setAddress(address);
+        bean.setResourceClass(BookStore.class);
+        BookStore store = bean.create(BookStore.class);
+        store.emptypostNoPath();
+        assertEquals(204, WebClient.client(store).getResponse().getStatus());
+    }
+    
     @Test
     public void testGetBookByEncodedQuery() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/bookquery?"
@@ -416,7 +647,21 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                       + "parameter, static valueOf(String) or fromString(String) methods",
                       "*/*", 500);
     }
-    
+
+    @Test
+    public void testWrongContentType() throws Exception {
+        // can't use WebClient here because WebClient plays around with the Content-Type
+        // (and makes sure it's syntactically correct) before sending it to the server
+        String endpointAddress = "http://localhost:" + PORT + "/bookstore/unsupportedcontenttype";
+        URL url = new URL(endpointAddress);
+        HttpURLConnection urlConnection = (HttpURLConnection)url.openConnection();
+        urlConnection.setReadTimeout(30000); // 30 seconds tops
+        urlConnection.setConnectTimeout(30000); // 30 second tops
+        urlConnection.addRequestProperty("Content-Type", "MissingSeparator");
+        urlConnection.setRequestMethod("POST");
+        assertEquals(415, urlConnection.getResponseCode());
+    }
+
     @Test
     public void testExceptionDuringConstruction() throws Exception {
         getAndCompare("http://localhost:" + PORT + "/bookstore/exceptionconstruction?p=1",
@@ -559,6 +804,32 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         assertEquals(200, r.getStatus());
     }
     
+    @Test
+    public void testGetBookFromResponseWithProxy() throws Exception {
+        ResponseReader reader = new ResponseReader();
+        reader.setEntityClass(Book.class);
+        
+        BookStore bs = JAXRSClientFactory.create("http://localhost:" + PORT, 
+                                                 BookStore.class,
+                                                 Collections.singletonList(reader));
+        Response r = bs.getGenericResponseBook("123");
+        assertEquals(200, r.getStatus());
+        Book book = (Book)r.getEntity();
+        assertEquals(123L, book.getId());
+    }
+    
+    @Test
+    public void testGetBookFromResponseWithWebClient() throws Exception {
+        String address = "http://localhost:" + PORT + "/bookstore/genericresponse/123";
+        WebClient wc = WebClient.create(address, 
+                                        Collections.singletonList(
+                                        new ResponseReader(Book.class)));
+        Response r = wc.accept("application/xml").get();
+        assertEquals(200, r.getStatus());
+        Book book = (Book)r.getEntity();
+        assertEquals(123L, book.getId());
+    }
+    
     @Test
     public void testUpdateWithProxy() throws Exception {
         BookStore bs = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
@@ -598,6 +869,13 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         
     }
     
+    @Test
+    public void testGetBook123CGLIB() throws Exception {
+        getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/books/123/cglib",
+                               "resources/expected_get_book123.txt",
+                               "application/xml", 200);
+    }
+    
     @Test
     public void testGetBook123() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/books/123",
@@ -672,6 +950,14 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testGetBookByMatrixParametersInTheMiddle() throws Exception {
+        getAndCompareAsStrings(
+            "http://localhost:" + PORT + "/bookstore/segment;first=12;second=3/matrix-middle",
+            "resources/expected_get_book123.txt",
+            "application/xml", 200);
+    }
+    
     @Test
     public void testGetBookByHeader() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/bookheaders",
@@ -700,6 +986,29 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testEchoBookElement() throws Exception {
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
+        JAXBElement<Book> element = store.echoBookElement(new JAXBElement<Book>(new QName("", "Book"),
+                                     Book.class,
+                                     new Book("CXF", 123L)));
+        Book book = element.getValue();
+        assertEquals(123L, book.getId());
+        assertEquals("CXF", book.getName());
+    }
+    
+    @Test
+    public void testEchoBookElementWildcard() throws Exception {
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
+        JAXBElement<? super Book> element = store.echoBookElementWildcard(
+                                        new JAXBElement<Book>(new QName("", "Book"),
+                                        Book.class,
+                                        new Book("CXF", 123L)));
+        Book book = (Book)element.getValue();
+        assertEquals(123L, book.getId());
+        assertEquals("CXF", book.getName());
+    }
+    
     @Test
     public void testGetBookAdapter() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/books/adapter",
@@ -716,13 +1025,28 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         provider.setOutTransformElements(outMap);
         WebClient wc = WebClient.create("http://localhost:" + PORT + "/bookstore/books/adapter-list",
                                         Collections.singletonList(provider));
-        
         Collection<? extends Book> books = wc.type("application/xml").accept("application/xml")
             .postAndGetCollection(new Books(new Book("CXF", 123L)), Book.class);
         assertEquals(1, books.size());
         assertEquals(123L, books.iterator().next().getId());
     }
     
+    @Test
+    public void testPostGetBookAdapterListJSON() throws Exception {
+        JAXBElementProvider provider = new JAXBElementProvider();
+        Map<String, String> outMap = new HashMap<String, String>();
+        outMap.put("Books", "CollectionWrapper");
+        outMap.put("books", "Book");
+        provider.setOutTransformElements(outMap);
+        WebClient wc = WebClient.create("http://localhost:" + PORT + "/bookstore/books/adapter-list",
+                                        Collections.singletonList(provider));
+        WebClient.getConfig(wc).getHttpConduit().getClient().setReceiveTimeout(10000000);
+        Response r = wc.type("application/xml").accept("application/json")
+            .post(new Books(new Book("CXF", 123L)));
+        assertEquals("{\"Book\":[{\"id\":123,\"name\":\"CXF\"}]}",
+                     IOUtils.readStringFromStream((InputStream)r.getEntity()));
+    }
+    
     @Test
     public void testGetBookAdapterInterface() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/books/interface/adapter",
@@ -730,6 +1054,39 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testGetBookAdapterInterfaceList() throws Exception {
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
+        List<? extends BookInfoInterface> list =  store.getBookAdapterInterfaceList();
+        assertEquals(1, list.size());
+        BookInfoInterface info = list.get(0);
+        assertEquals(123L, info.getId());
+    }
+    
+    @Test
+    public void testGetBookAdapterInterfaceProxy() throws Exception {
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
+        WebClient.getConfig(store).getHttpConduit().getClient().setReceiveTimeout(10000000L);
+        BookInfoInterface info = store.getBookAdapterInterface();
+        assertEquals(123L, info.getId());
+    }
+    
+    @Test
+    public void testGetBookAdapterInfoList() throws Exception {
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
+        List<? extends BookInfo> list =  store.getBookAdapterList();
+        assertEquals(1, list.size());
+        BookInfo info = list.get(0);
+        assertEquals(123L, info.getId());
+    }
+    
+    @Test
+    public void testGetBookAdapterInfoProxy() throws Exception {
+        BookStore store = JAXRSClientFactory.create("http://localhost:" + PORT, BookStore.class);
+        BookInfo info = store.getBookAdapter();
+        assertEquals(123L, info.getId());
+    }
+    
     @Test
     public void testGetBook123FromSub() throws Exception {
         getAndCompareAsStrings("http://localhost:" + PORT + "/bookstore/interface/subresource",
@@ -820,6 +1177,19 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         }
     }
     
+    @Test
+    public void testAddBookEmptyContent() throws Exception {
+        Response r = WebClient.create("http://localhost:" + PORT + "/bookstore/books").post(null);
+        assertEquals(400, r.getStatus());
+    }
+    
+    @Test
+    public void testAddBookEmptyContentWithNullable() throws Exception {
+        Book defaultBook = 
+            WebClient.create("http://localhost:" + PORT + "/bookstore/books/null").post(null, Book.class);
+        assertEquals("Default Book", defaultBook.getName());
+    }
+    
     @Test
     public void testAddBook() throws Exception {
         doAddBook("http://localhost:" + PORT + "/bookstore/books");               
@@ -877,39 +1247,14 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         try {
             int result = httpclient.executeMethod(put);
             assertEquals(200, result);
+            InputStream expected = getClass().getResourceAsStream("resources/expected_update_book.txt");
+            assertEquals(getStringFromInputStream(expected), 
+                         getStringFromInputStream(put.getResponseBodyAsStream()));
         } finally {
             // Release current connection to the connection pool once you are
             // done
             put.releaseConnection();
         }
-
-        // Verify result
-        endpointAddress = "http://localhost:" + PORT + "/bookstore/books/123";
-        URL url = new URL(endpointAddress);
-        URLConnection connect = url.openConnection();
-        connect.addRequestProperty("Accept", "application/xml");
-        InputStream in = connect.getInputStream();
-        assertNotNull(in);
-
-        InputStream expected = getClass().getResourceAsStream("resources/expected_update_book.txt");
-
-        assertEquals(getStringFromInputStream(expected), getStringFromInputStream(in));
-
-        // Roll back changes:
-        File input1 = new File(getClass().getResource("resources/expected_get_book123.txt").toURI());
-        PutMethod put1 = new PutMethod(endpointAddress);
-        RequestEntity entity1 = new FileRequestEntity(input1, "text/xml; charset=ISO-8859-1");
-        put1.setRequestEntity(entity1);
-        HttpClient httpclient1 = new HttpClient();
-
-        try {
-            int result = httpclient1.executeMethod(put);
-            assertEquals(200, result);
-        } finally {
-            // Release current connection to the connection pool once you are
-            // done
-            put1.releaseConnection();
-        }
     }  
     
     @Test
@@ -950,39 +1295,14 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         try {
             int result = httpclient.executeMethod(put);
             assertEquals(200, result);
+            InputStream expected = getClass().getResourceAsStream("resources/expected_update_book.txt");
+            assertEquals(getStringFromInputStream(expected), 
+                         getStringFromInputStream(put.getResponseBodyAsStream()));
         } finally {
             // Release current connection to the connection pool once you are
             // done
             put.releaseConnection();
         }
-
-        // Verify result
-        endpointAddress = "http://localhost:" + PORT + "/bookstore/books/123";
-        URL url = new URL(endpointAddress);
-        URLConnection connection = url.openConnection();
-        connection.addRequestProperty("Accept", "application/xml");
-        InputStream in = connection.getInputStream();
-        assertNotNull(in);
-
-        InputStream expected = getClass().getResourceAsStream("resources/expected_update_book.txt");
-
-        assertEquals(getStringFromInputStream(expected), getStringFromInputStream(in));
-
-        // Roll back changes:
-        File input1 = new File(getClass().getResource("resources/expected_get_book123.txt").toURI());
-        PutMethod put1 = new PutMethod(endpointAddress);
-        RequestEntity entity1 = new FileRequestEntity(input1, "text/xml; charset=ISO-8859-1");
-        put1.setRequestEntity(entity1);
-        HttpClient httpclient1 = new HttpClient();
-
-        try {
-            int result = httpclient1.executeMethod(put);
-            assertEquals(200, result);
-        } finally {
-            // Release current connection to the connection pool once you are
-            // done
-            put1.releaseConnection();
-        }
     } 
     
     @Test
@@ -1115,8 +1435,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "resources/expected_get_cd.txt",
                                "application/xml", 200);
     }
-    
-    
+
     @Test
     public void testGetCDWithMultiContentTypesXML() throws Exception {
         
@@ -1147,9 +1466,9 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     public void testUriInfoMatchedResources() throws Exception {
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched-resources", 
-                      "[class org.apache.cxf.systest.jaxrs.BookStore, " 
+                      "[class org.apache.cxf.systest.jaxrs.Chapter, " 
                       + "class org.apache.cxf.systest.jaxrs.Book, "
-                      + "class org.apache.cxf.systest.jaxrs.Chapter]", 
+                      + "class org.apache.cxf.systest.jaxrs.BookStore]", 
                       "text/plain", "text/plain", 200);
     }
     
@@ -1157,17 +1476,17 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     public void testUriInfoMatchedResourcesWithObject() throws Exception {
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chaptersobject/sub/1/matched-resources", 
-                      "[class org.apache.cxf.systest.jaxrs.BookStore, " 
+                      "[class org.apache.cxf.systest.jaxrs.Chapter, " 
                       + "class org.apache.cxf.systest.jaxrs.Book, "
-                      + "class org.apache.cxf.systest.jaxrs.Chapter]", 
+                      + "class org.apache.cxf.systest.jaxrs.BookStore]", 
                       "text/plain", "text/plain", 200);
     }
 
     @Test
     public void testUriInfoMatchedUrisDecode() throws Exception {
-        String expected = "[/bookstore/booksubresource/123/, "
+        String expected = "[/bookstore/booksubresource/123/chapters/sub/1/matched!uris, "
                           + "/bookstore/booksubresource/123/chapters/sub/1/, "
-                          + "/bookstore/booksubresource/123/chapters/sub/1/matched!uris]";
+                          + "/bookstore/booksubresource/123/]";
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched%21uris?decode=true", 
                       expected, "text/plain", "text/plain", 200);
@@ -1176,16 +1495,75 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     @Test
     public void testUriInfoMatchedUrisNoDecode() throws Exception {
         //note '%21' instead of '!'
-        String expected = "[/bookstore/booksubresource/123/, "
+        String expected = "[/bookstore/booksubresource/123/chapters/sub/1/matched%21uris, "
             + "/bookstore/booksubresource/123/chapters/sub/1/, "
-            + "/bookstore/booksubresource/123/chapters/sub/1/matched%21uris]";
+            + "/bookstore/booksubresource/123/]";
         getAndCompare("http://localhost:" + PORT + "/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched%21uris?decode=false", 
                       expected,
                       "text/plain", "text/plain", 200);
     }
-    
-    private void getAndCompareAsStrings(String address, 
+
+    @Test
+    public void testQuotedHeaders() throws Exception {
+
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/quotedheaders";
+        Response r = WebClient.create(endpointAddress).get();
+
+        List<Object> header1 = r.getMetadata().get("SomeHeader1");
+        assertEquals(1, header1.size());
+        assertEquals("\"some text, some more text\"", header1.get(0));
+
+        List<Object> header2 = r.getMetadata().get("SomeHeader2");
+        assertEquals(3, header2.size());
+        assertEquals("\"some text\"", header2.get(0));
+        assertEquals("\"quoted,text\"", header2.get(1));
+        assertEquals("\"even more text\"", header2.get(2));
+
+        List<Object> header3 = r.getMetadata().get("SomeHeader3");
+        assertEquals(1, header3.size());
+        assertEquals("\"some text, some more text with inlined \"\"", header3.get(0));
+
+        List<Object> header4 = r.getMetadata().get("SomeHeader4");
+        assertEquals(1, header4.size());
+        assertEquals("\"\"", header4.get(0));
+
+    }
+
+    @Test
+    public void testBadlyQuotedHeaders() throws Exception {
+
+        String endpointAddress =
+            "http://localhost:" + PORT + "/bookstore/badlyquotedheaders";
+
+        String[] responses = new String[] {
+            "\"some text",
+            "\"some text, some more text with inlined \"",
+            "\"some te\\",
+        };
+
+        // technically speaking, for these test cases, the client should return an error
+        // however, servers do send bad data from time to time so we try to be forgiving
+        for (int i = 0; i < 3; i++) {
+            Response r = WebClient.create(endpointAddress).query("type", Integer.toString(i)).get();
+            assertEquals(responses[i], r.getMetadata().get("SomeHeader" + i).get(0));
+        }
+
+        // this test currently returns the WRONG result per RFC2616, however it is correct
+        // per the discussion in CXF-3518
+        Response r3 = WebClient.create(endpointAddress).query("type", "3").get();
+        List<Object> r3values = r3.getMetadata().get("SomeHeader3");
+        assertEquals(4, r3values.size());
+        assertEquals("some text", r3values.get(0));
+        assertEquals("\"other quoted\"", r3values.get(1));
+        assertEquals("text", r3values.get(2));
+        assertEquals("blah", r3values.get(3));
+
+    }
+
+
+    private void getAndCompareAsStrings(String address,
                                         String resourcePath,
                                         String acceptType,
                                         int status) throws Exception {
