@@ -50,8 +50,10 @@ import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.interceptor.FIStaxInInterceptor;
 import org.apache.cxf.interceptor.FIStaxOutInterceptor;
 import org.apache.cxf.interceptor.Fault;
+import org.apache.cxf.interceptor.Interceptor;
 import org.apache.cxf.interceptor.InterceptorProvider;
 import org.apache.cxf.io.CachedOutputStream;
+import org.apache.cxf.jaxrs.client.ClientConfiguration;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
 import org.apache.cxf.jaxrs.client.JAXRSClientFactoryBean;
 import org.apache.cxf.jaxrs.client.ResponseExceptionMapper;
@@ -65,6 +67,8 @@ import org.apache.cxf.phase.AbstractPhaseInterceptor;
 import org.apache.cxf.phase.Phase;
 import org.apache.cxf.systest.jaxrs.jaxws.BookSoapService;
 import org.apache.cxf.systest.jaxrs.jaxws.BookStoreJaxrsJaxws;
+import org.apache.cxf.systest.jaxrs.jaxws.BookStoreSoapRestFastInfoset2;
+import org.apache.cxf.systest.jaxrs.jaxws.BookStoreSoapRestFastInfoset3;
 import org.apache.cxf.systest.jaxrs.jaxws.HelloWorld;
 import org.apache.cxf.systest.jaxrs.jaxws.User;
 import org.apache.cxf.systest.jaxrs.jaxws.UserImpl;
@@ -106,6 +110,24 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         useHelloService(hw);
     }
     
+    @Test
+    public void testHelloSoapCustomDataBinding() throws Exception {
+        final QName serviceName = new QName("http://hello.com", "HelloWorld");
+        final QName portName = new QName("http://hello.com", "HelloWorldPort");
+        final String address = "http://localhost:" + PORT + "/test/services/hello-soap-databinding";
+        
+        Service service = Service.create(serviceName);
+        service.addPort(portName, SOAPBinding.SOAP11HTTP_BINDING, address);
+    
+        HelloWorld hw = service.getPort(HelloWorld.class); 
+    
+        User user = new UserImpl("Barry");
+        User user2 = hw.echoUser(user);
+        
+        assertNotSame(user, user2);
+        assertEquals("Barry", user.getName());
+    }
+    
     private void useHelloService(HelloWorld service) {
         assertEquals("Hello Barry", service.sayHi("Barry"));
         assertEquals("Hello Fred", service.sayHiToUser(new UserImpl("Fred")));
@@ -172,6 +194,72 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         assertEquals(b2.getId(), b.getId());
     }
     
+    @Test
+    public void testPostGetBookFastinfosetProxy() throws Exception {
+        
+        JAXBElementProvider p = new JAXBElementProvider();
+        p.setConsumeMediaTypes(Collections.singletonList("application/fastinfoset"));
+        p.setProduceMediaTypes(Collections.singletonList("application/fastinfoset"));
+        
+        BookStoreJaxrsJaxws client = JAXRSClientFactory.create(
+                                  "http://localhost:" + PORT + "/test/services/rest4",
+                                  BookStoreSoapRestFastInfoset2.class,
+                                  Collections.singletonList(p));
+        
+        Book b = new Book("CXF", 1L);
+        
+        Book b2 = client.addFastinfoBook(b);
+        
+        assertEquals(b2.getName(), b.getName());
+        assertEquals(b2.getId(), b.getId());
+        
+        checkFiInterceptors(WebClient.getConfig(client));
+    }
+    
+    @Test
+    public void testPostGetBookFastinfosetProxyInterceptors() throws Exception {
+        
+        JAXBElementProvider p = new JAXBElementProvider();
+        p.setConsumeMediaTypes(Collections.singletonList("application/fastinfoset"));
+        p.setProduceMediaTypes(Collections.singletonList("application/fastinfoset"));
+        
+        BookStoreJaxrsJaxws client = JAXRSClientFactory.create(
+                                  "http://localhost:" + PORT + "/test/services/rest5",
+                                  BookStoreSoapRestFastInfoset3.class,
+                                  Collections.singletonList(p));
+        
+        Book b = new Book("CXF", 1L);
+        
+        // Just to make sure it is enforced
+        Map<String, Object> props = WebClient.getConfig(client).getResponseContext();
+        props.put(FIStaxOutInterceptor.FI_ENABLED, Boolean.TRUE);
+        
+        Book b2 = client.addFastinfoBook(b);
+        
+        assertEquals(b2.getName(), b.getName());
+        assertEquals(b2.getId(), b.getId());
+        
+        checkFiInterceptors(WebClient.getConfig(client));
+        
+    }
+    
+    private void checkFiInterceptors(ClientConfiguration cfg) {
+        int count = 0;
+        for (Interceptor<?> in : cfg.getInInterceptors()) {
+            if (in instanceof FIStaxInInterceptor) {
+                count++;
+                break;
+            }
+        }
+        for (Interceptor<?> in : cfg.getOutInterceptors()) {
+            if (in instanceof FIStaxOutInterceptor) {
+                count++;
+                break;
+            }
+        }
+        assertEquals("In and Out FastInfoset interceptors are expected", 2, count);
+    }
+    
     @Test
     public void testGetBook123ServletResponse() throws Exception {
         
@@ -437,8 +525,8 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         WebClient wc = WebClient.create(baseAddress);
         MultivaluedMap<String, Object> map = new MetadataMap<String, Object>();
         map.putSingle("id", "679");
-        map.putSingle("name", "CXF in Action - ");
-        map.putSingle("nameid", "679");
+        map.add("name", "CXF in Action - ");
+        map.add("name", "679");
         Book b = readBook((InputStream)wc.accept("application/xml")
                           .form((Map<String, List<Object>>)map).getEntity());
         assertEquals(679, b.getId());
@@ -453,7 +541,7 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         WebClient wc = WebClient.create(baseAddress);
         Form f = new Form();
         f.set("id", "679").set("name", "CXF in Action - ")
-            .set("nameid", "679");
+            .set("name", "679");
         Book b = readBook((InputStream)wc.accept("application/xml")
                           .form(f).getEntity());
         assertEquals(679, b.getId());
@@ -467,7 +555,10 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         BookStoreJaxrsJaxws proxy = JAXRSClientFactory.create(baseAddress,
                                                                   BookStoreJaxrsJaxws.class);
         BookSubresource bs = proxy.getBookSubresource("679");
-        Book b = bs.getTheBook3("679", "CXF in Action - ", new Integer(679));
+        List<String> parts = new ArrayList<String>();
+        parts.add("CXF in Action - ");
+        parts.add(Integer.toString(679));
+        Book b = bs.getTheBook3("679", parts);
         assertEquals(679, b.getId());
         assertEquals("CXF in Action - 679", b.getName());
     }
@@ -556,12 +647,12 @@ public class JAXRSSoapBookTest extends AbstractBusClientServerTestBase {
         assertEquals(200, WebClient.create(
             "http://localhost:" + PORT + "/test/services/rest2?_wadl&type=xml").get().getStatus());
         assertFalse(listings.contains("http://localhost:" + PORT + "/test/services/rest3?_wadl&type=xml"));
-        assertEquals(401, WebClient.create(
-            "http://localhost:" + PORT + "/test/services/rest3?_wadl&type=xml").get().getStatus());
-        
-         
-        
         assertFalse(listings.contains("Atom Log Feed"));
+    
+        WebClient webClient = 
+            WebClient.create("http://localhost:" + PORT + "/test/services/rest3?_wadl&type=xml");
+        WebClient.getConfig(webClient).getHttpConduit().getClient().setReceiveTimeout(1000000);
+        assertEquals(401, webClient.get().getStatus());
     }
     
     @Test
