@@ -383,6 +383,29 @@ public class ClientServerTest extends AbstractBusClientServerTestBase {
             throw (Exception)ex.getCause();
         }
     } 
+    
+    @Test
+    public void testAsyncDiscardProxy() throws Exception {
+        URL wsdl = getClass().getResource("/wsdl/hello_world.wsdl");
+        assertNotNull(wsdl);
+        
+        SOAPService service = new SOAPService(wsdl, serviceName);
+        
+        assertNotNull(service);
+        
+        Greeter greeter = service.getPort(portName, Greeter.class);
+        
+        assertNotNull(service);
+        updateAddressPort(greeter, PORT);
+
+        Response<GreetMeLaterResponse> r1 = greeter.greetMeLaterAsync(3000);
+        greeter = null;
+        service = null;
+        System.gc();
+        System.gc();
+        System.gc();
+        assertEquals("Hello, finally!", r1.get().getResponseType());
+    }
 
     @Test
     public void testAsyncPollingCall() throws Exception {
@@ -1003,21 +1026,4 @@ public class ClientServerTest extends AbstractBusClientServerTestBase {
         ins.close();
     }
     
-    @Test
-    public void testGetPortWithoutPortName() throws Exception {
-        SOAPService service = new SOAPService();
-        assertNotNull(service);
-        Greeter greeter = service.getPort(Greeter.class);
-        updateAddressPort(greeter, PORT);
-        try {
-            greeter.greetMe("test");           
-            String reply = greeter.sayHi();
-            assertNotNull("no response received from service", reply);
-            assertEquals("Bonjour", reply);
-        } catch (UndeclaredThrowableException ex) {
-            throw (Exception)ex.getCause();
-        }
-    }
-    
-    
 }
