@@ -21,10 +21,22 @@ package org.apache.cxf.systest.ws.addr_fromjava;
 
 import java.io.ByteArrayOutputStream;
 import java.net.URL;
+import java.util.List;
 import java.util.Map;
+
 import javax.xml.ws.BindingProvider;
 import javax.xml.ws.soap.SOAPFaultException;
 
+
+import org.apache.cxf.binding.soap.SoapMessage;
+import org.apache.cxf.binding.soap.interceptor.AbstractSoapInterceptor;
+import org.apache.cxf.binding.soap.interceptor.ReadHeadersInterceptor;
+import org.apache.cxf.endpoint.Client;
+import org.apache.cxf.feature.LoggingFeature;
+import org.apache.cxf.frontend.ClientProxy;
+import org.apache.cxf.headers.Header;
+import org.apache.cxf.interceptor.Fault;
+import org.apache.cxf.phase.Phase;
 import org.apache.cxf.systest.ws.AbstractWSATestBase;
 import org.apache.cxf.systest.ws.addr_fromjava.client.AddNumberImpl;
 import org.apache.cxf.systest.ws.addr_fromjava.client.AddNumberImplService;
@@ -35,7 +47,8 @@ import org.junit.BeforeClass;
 import org.junit.Test;
 
 public class WSAFromJavaTest extends AbstractWSATestBase {
-
+    static final String PORT = allocatePort(Server.class);
+    
     @Before
     public void setUp() throws Exception {
         createBus();
@@ -43,7 +56,7 @@ public class WSAFromJavaTest extends AbstractWSATestBase {
 
     @BeforeClass
     public static void startServers() throws Exception {
-        assertTrue("server did not launch correctly", launchServer(Server.class));
+        assertTrue("server did not launch correctly", launchServer(Server.class, true));
     }
 
     @Test
@@ -63,7 +76,7 @@ public class WSAFromJavaTest extends AbstractWSATestBase {
     }
 
     @Test
-    public void testAddNumbersFault() {
+    public void testAddNumbersFault() throws Exception {
         ByteArrayOutputStream input = setupInLogging();
         ByteArrayOutputStream output = setupOutLogging();
 
@@ -104,7 +117,7 @@ public class WSAFromJavaTest extends AbstractWSATestBase {
     }
 
     @Test
-    public void testAddNumbers3Fault() {
+    public void testAddNumbers3Fault() throws Exception {
         ByteArrayOutputStream input = setupInLogging();
         ByteArrayOutputStream output = setupOutLogging();
 
@@ -144,14 +157,16 @@ public class WSAFromJavaTest extends AbstractWSATestBase {
         assertTrue(output.toString().indexOf("SOAPAction=[\"cxf\"]") != -1);
     }
 
-    private AddNumberImpl getPort() {
+    private AddNumberImpl getPort() throws Exception {
         URL wsdl = getClass().getResource("/wsdl_systest_wsspec/add_numbers-fromjava.wsdl");
         assertNotNull("WSDL is null", wsdl);
 
         AddNumberImplService service = new AddNumberImplService(wsdl);
         assertNotNull("Service is null ", service);
 
-        return service.getAddNumberImplPort();
+        AddNumberImpl port = service.getAddNumberImplPort();
+        updateAddressPort(port, PORT);
+        return port;
     }
     
     @Test 
@@ -169,4 +184,61 @@ public class WSAFromJavaTest extends AbstractWSATestBase {
             assertTrue(e.getMessage().contains("Unexpected wrapper"));
         }
     }
+    
+    @Test
+    public void testFaultFromNonAddressService() throws Exception {
+        new LoggingFeature().initialize(this.getBus());
+        AddNumberImpl port = getPort();
+        java.util.Map<String, Object> requestContext = ((BindingProvider)port).getRequestContext();
+        requestContext.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
+                           "http://localhost:" + PORT + "/AddNumberImplPort-noaddr");
+        
+        long start = System.currentTimeMillis();
+        port.addNumbers(1, 2);
+        try {
+            port.addNumbers3(-1, -1);
+        } catch (Exception ex) {
+            //ignore, expected
+        }
+        long end = System.currentTimeMillis();
+        assertTrue((end - start) < 50000);
+    }
+    
+    static class RemoveRelatesToHeaderInterceptor extends AbstractSoapInterceptor {
+        public RemoveRelatesToHeaderInterceptor() {
+            super(Phase.READ);
+            addAfter(ReadHeadersInterceptor.class.getName());
+        }
+        public void handleMessage(SoapMessage message) throws Fault {
+            List<Header> headers = message.getHeaders();
+            Header h2 = null;
+            for (Header h : headers) {
+                if ("RelatesTo".equals(h.getName().getLocalPart())) {
+                    h2 = h;
+                }
+            }
+            headers.remove(h2);
+        }
+    }
+    
+    @Test
+    public void testNoRelatesToHeader() throws Exception {
+        new LoggingFeature().initialize(this.getBus());
+        AddNumberImpl port = getPort();
+
+        Client c = ClientProxy.getClient(port);
+        c.getInInterceptors().add(new RemoveRelatesToHeaderInterceptor());
+        
+        
+        long start = System.currentTimeMillis();
+        port.addNumbers(1, 2);
+        try {
+            port.addNumbers3(-1, -1);
+        } catch (Exception ex) {
+            //ignore, expected
+        }
+        long end = System.currentTimeMillis();
+        assertTrue((end - start) < 50000);
+    }    
+    
 }
\ No newline at end of file
