@@ -19,12 +19,14 @@
 package org.apache.cxf.tools.wsdlto.jaxws;
 
 import java.io.File;
+import java.io.ObjectStreamClass;
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.lang.reflect.ParameterizedType;
 import java.lang.reflect.Type;
 import java.util.Arrays;
+import java.util.List;
 
 import javax.jws.HandlerChain;
 import javax.jws.Oneway;
@@ -44,6 +46,7 @@ import org.apache.cxf.tools.common.ToolConstants;
 import org.apache.cxf.tools.common.ToolException;
 import org.apache.cxf.tools.util.AnnotationUtil;
 import org.apache.cxf.tools.wsdlto.AbstractCodeGenTest;
+
 import org.junit.Test;
 
 public class CodeGenTest extends AbstractCodeGenTest {
@@ -1331,5 +1334,94 @@ public class CodeGenTest extends AbstractCodeGenTest {
         WebParam wp = (WebParam)m[0].getParameterAnnotations()[1][0];
         assertTrue(wp.header());
     }
+    
+    @Test
+    public void testCXFNotType() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        env.put(ToolConstants.CFG_USE_FQCN_FAULT_SERIAL_VERSION_UID, Boolean.TRUE);
+        processor.setContext(env);
+        processor.execute();
+        
+        File sayHiResponseFile =
+            new File(output, "org/apache/cxf/w2j/hello_world_soap_http/types/SayHiResponse");
+        assertFalse(sayHiResponseFile.exists());
+        File greetMeResponseFile =
+            new File(output, "org/apache/cxf/w2j/hello_world_soap_http/types/GreetMeResponse");
+        assertFalse(greetMeResponseFile.exists());
+    }
+
+    @Test
+    public void testCXF2808() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        env.put(ToolConstants.CFG_USE_FQCN_FAULT_SERIAL_VERSION_UID, Boolean.TRUE);
+        processor.setContext(env);
+        processor.execute();
+
+        File faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/NoSuchCodeLitFault.java");
+        assertTrue(faultFile.exists());
+        faultFile = new File(output, "org/apache/cxf/w2j/hello_world_soap_http/BadRecordLitFault.java");
+        assertTrue(faultFile.exists());
+
+        Class<?> fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.NoSuchCodeLitFault");
+        assertEquals(fault.getName().hashCode(), ObjectStreamClass.lookup(fault).getSerialVersionUID());
+        fault = classLoader.loadClass("org.apache.cxf.w2j.hello_world_soap_http.BadRecordLitFault");
+        assertEquals(fault.getName().hashCode(), ObjectStreamClass.lookup(fault).getSerialVersionUID());
+    }
+    
+    
+    @Test
+    public void testExtensionWrapper() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL,
+                getLocation("/wsdl2java_wsdl/cxf2193/hello_world_extension_wrapped.wsdl"));
+        processor.setContext(env);
+        processor.execute();
+
+        File infFile = new File(output, "org/apache/cxf/w2j/extension_wrapped/Greeter.java");
+        assertTrue(infFile.exists());
+
+        Class<?> interfaceClass = classLoader.loadClass("org.apache.cxf.w2j.extension_wrapped.Greeter");
+
+        Method method = interfaceClass.getMethod("greetMe", new Class[] {
+            Holder.class, Holder.class, Holder.class, Holder.class, Holder.class
+        });
+        assertTrue("greetMe operation is NOT generated correctly as excepted", method != null);
+        RequestWrapper reqWrapper = method.getAnnotation(RequestWrapper.class);
+        assertNotNull("@RequestWrapper is expected", reqWrapper);
+    }
+    @Test
+    public void testJavaDoc() throws Exception {
+        env.put(ToolConstants.CFG_WSDLURL, getLocation("/wsdl2java_wsdl/hello_world.wsdl"));
+        processor.setContext(env);
+        processor.execute();
+
+        List<String> results1 = FileUtils.readLines(new File(output.getCanonicalPath(),
+            "org/apache/cxf/w2j/hello_world_soap_http/Greeter.java"));
+
+        System.out.println(results1);
+        assertTrue(results1.contains(" * porttype documentation"));
+        assertTrue(results1.contains("     * porttype op documentation"));
+
+        List<String> results2 = FileUtils.readLines(new File(output.getCanonicalPath(),
+            "org/apache/cxf/w2j/hello_world_soap_http/SOAPServiceTest1.java"));
+
+        boolean match1 = false;
+        boolean match2 = false;
+        boolean match3 = false;
+        for (String str : results2) {
+            if (str.contains("service documentation")) {
+                match1 = true;
+            }
+            if (str.contains("port1 documentation")) {
+                match2 = true;
+            }
+            if (str.contains("port2 documentation")) {
+                match3 = true;
+            }
+        }
+        assertTrue(results2.toString(), match1);
+        assertTrue(results2.toString(), match2);
+        assertTrue(results2.toString(), match3);
+
+    }
 
 }
